# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import requests
import uuid

from urllib import parse as urlparse

from keystoneauth1.identity import v3
from keystoneauth1 import session
from keystoneclient.tests.unit import client_fixtures
from keystoneclient.tests.unit import utils
from keystoneclient.v3 import client


def parameterize(ref):
    """Rewrite attributes to match the kwarg naming convention in client.

    >>> parameterize({'project_id': 0})
    {'project': 0}

    """
    params = ref.copy()
    for key in ref:
        if key[-3:] == '_id':
            params.setdefault(key[:-3], params.pop(key))
    return params


class UnauthenticatedTestCase(utils.TestCase):
    """Class used as base for unauthenticated calls."""

    TEST_ROOT_URL = 'http://127.0.0.1:5000/'
    TEST_URL = '%s%s' % (TEST_ROOT_URL, 'v3')
    TEST_ROOT_ADMIN_URL = 'http://127.0.0.1:35357/'
    TEST_ADMIN_URL = '%s%s' % (TEST_ROOT_ADMIN_URL, 'v3')


class TestCase(UnauthenticatedTestCase):

    TEST_ADMIN_IDENTITY_ENDPOINT = "http://127.0.0.1:35357/v3"
    TEST_PUBLIC_IDENTITY_ENDPOINT = "http://127.0.0.1:5000/v3"

    TEST_SERVICE_CATALOG = [{
        "endpoints": [{
            "url": "http://cdn.admin-nets.local:8774/v1.0/",
            "region": "RegionOne",
            "interface": "public"
        }, {
            "url": "http://127.0.0.1:8774/v1.0",
            "region": "RegionOne",
            "interface": "internal"
        }, {
            "url": "http://cdn.admin-nets.local:8774/v1.0",
            "region": "RegionOne",
            "interface": "admin"
        }],
        "type": "nova_compat"
    }, {
        "endpoints": [{
            "url": "http://nova/novapi/public",
            "region": "RegionOne",
            "interface": "public"
        }, {
            "url": "http://nova/novapi/internal",
            "region": "RegionOne",
            "interface": "internal"
        }, {
            "url": "http://nova/novapi/admin",
            "region": "RegionOne",
            "interface": "admin"
        }],
        "type": "compute"
    }, {
        "endpoints": [{
            "url": "http://glance/glanceapi/public",
            "region": "RegionOne",
            "interface": "public"
        }, {
            "url": "http://glance/glanceapi/internal",
            "region": "RegionOne",
            "interface": "internal"
        }, {
            "url": "http://glance/glanceapi/admin",
            "region": "RegionOne",
            "interface": "admin"
        }],
        "type": "image",
        "name": "glance"
    }, {
        "endpoints": [{
            "url": "http://127.0.0.1:5000/v3",
            "region": "RegionOne",
            "interface": "public"
        }, {
            "url": "http://127.0.0.1:5000/v3",
            "region": "RegionOne",
            "interface": "internal"
        }, {
            "url": TEST_ADMIN_IDENTITY_ENDPOINT,
            "region": "RegionOne",
            "interface": "admin"
        }],
        "type": "identity"
    }, {
        "endpoints": [{
            "url": "http://swift/swiftapi/public",
            "region": "RegionOne",
            "interface": "public"
        }, {
            "url": "http://swift/swiftapi/internal",
            "region": "RegionOne",
            "interface": "internal"
        }, {
            "url": "http://swift/swiftapi/admin",
            "region": "RegionOne",
            "interface": "admin"
        }],
        "type": "object-store"
    }]

    def stub_auth(self, subject_token=None, **kwargs):

        if not subject_token:
            subject_token = self.TEST_TOKEN

        try:
            response_list = kwargs['response_list']
        except KeyError:
            headers = kwargs.setdefault('headers', {})
            headers['X-Subject-Token'] = subject_token
        else:
            for resp in response_list:
                headers = resp.setdefault('headers', {})
                headers['X-Subject-Token'] = subject_token

        self.stub_url('POST', ['auth', 'tokens'], **kwargs)


class ClientTestCase(utils.ClientTestCaseMixin, TestCase):

    ORIGINAL_CLIENT_TYPE = 'original'
    KSC_SESSION_CLIENT_TYPE = 'ksc-session'
    KSA_SESSION_CLIENT_TYPE = 'ksa-session'

    scenarios = [
        (
            ORIGINAL_CLIENT_TYPE, {
                'client_fixture_class': client_fixtures.OriginalV3,
                'client_type': ORIGINAL_CLIENT_TYPE
            }
        ),
        (
            KSC_SESSION_CLIENT_TYPE, {
                'client_fixture_class': client_fixtures.KscSessionV3,
                'client_type': KSC_SESSION_CLIENT_TYPE
            }
        ),
        (
            KSA_SESSION_CLIENT_TYPE, {
                'client_fixture_class': client_fixtures.KsaSessionV3,
                'client_type': KSA_SESSION_CLIENT_TYPE
            }
        )

    ]

    @property
    def is_original_client(self):
        return self.client_type == self.ORIGINAL_CLIENT_TYPE

    @property
    def is_session_client(self):
        return self.client_type in (self.KSC_SESSION_CLIENT_TYPE,
                                    self.KSA_SESSION_CLIENT_TYPE)


class CrudTests(object):
    key = None
    collection_key = None
    model = None
    manager = None
    path_prefix = None

    def new_ref(self, **kwargs):
        kwargs.setdefault('id', uuid.uuid4().hex)
        kwargs.setdefault(uuid.uuid4().hex, uuid.uuid4().hex)
        return kwargs

    def encode(self, entity):
        if isinstance(entity, dict):
            return {self.key: entity}
        if isinstance(entity, list):
            return {self.collection_key: entity}
        raise NotImplementedError('Are you sure you want to encode that?')

    def stub_entity(self, method, parts=None, entity=None, id=None, **kwargs):
        if entity:
            entity = self.encode(entity)
            kwargs['json'] = entity

        if not parts:
            parts = [self.collection_key]

            if self.path_prefix:
                parts.insert(0, self.path_prefix)

        if id:
            if not parts:
                parts = []

            parts.append(id)

        self.stub_url(method, parts=parts, **kwargs)

    def assertEntityRequestBodyIs(self, entity):
        self.assertRequestBodyIs(json=self.encode(entity))

    def test_create(self, ref=None, req_ref=None):
        deprecations = self.useFixture(client_fixtures.Deprecations())
        deprecations.expect_deprecations()
        ref = ref or self.new_ref()
        manager_ref = ref.copy()
        manager_ref.pop('id')

        # req_ref argument allows you to specify a different
        # signature for the request when the manager does some
        # conversion before doing the request (e.g. converting
        # from datetime object to timestamp string)
        if req_ref:
            req_ref = req_ref.copy()
        else:
            req_ref = ref.copy()
            req_ref.pop('id')

        self.stub_entity('POST', entity=req_ref, status_code=201)

        returned = self.manager.create(**parameterize(manager_ref))
        self.assertIsInstance(returned, self.model)
        for attr in req_ref:
            self.assertEqual(
                getattr(returned, attr),
                req_ref[attr],
                'Expected different %s' % attr)
        self.assertEntityRequestBodyIs(req_ref)

        # The entity created here may be used in other test cases
        return returned

    def test_get(self, ref=None):
        ref = ref or self.new_ref()

        self.stub_entity('GET', id=ref['id'], entity=ref)

        returned = self.manager.get(ref['id'])
        self.assertIsInstance(returned, self.model)
        for attr in ref:
            self.assertEqual(
                getattr(returned, attr),
                ref[attr],
                'Expected different %s' % attr)

    def _get_expected_path(self, expected_path=None):
        if not expected_path:
            if self.path_prefix:
                expected_path = 'v3/%s/%s' % (self.path_prefix,
                                              self.collection_key)
            else:
                expected_path = 'v3/%s' % self.collection_key

        return expected_path

    def test_list_by_id(self, ref=None, **filter_kwargs):
        """Test ``entities.list(id=x)`` being rewritten as ``GET /v3/entities/x``.  # noqa

        This tests an edge case of each manager's list() implementation, to
        ensure that it "does the right thing" when users call ``.list()``
        when they should have used ``.get()``.

        """
        if 'id' not in filter_kwargs:
            ref = ref or self.new_ref()
            filter_kwargs['id'] = ref['id']

        self.assertRaises(TypeError, self.manager.list, **filter_kwargs)

    def test_list(self, ref_list=None, expected_path=None,
                  expected_query=None, **filter_kwargs):
        ref_list = ref_list or [self.new_ref(), self.new_ref()]
        expected_path = self._get_expected_path(expected_path)

        self.requests_mock.get(urlparse.urljoin(self.TEST_URL, expected_path),
                               json=self.encode(ref_list))

        returned_list = self.manager.list(**filter_kwargs)
        self.assertEqual(len(ref_list), len(returned_list))
        [self.assertIsInstance(r, self.model) for r in returned_list]

        qs_args = self.requests_mock.last_request.qs
        qs_args_expected = expected_query or filter_kwargs
        for key, value in qs_args_expected.items():
            self.assertIn(key, qs_args)
            # The querystring value is a list. Note we convert the value to a
            # string and lower, as the query string is always a string and the
            # filter_kwargs may contain non-string values, for example a
            # boolean, causing the comaprison to fail.
            self.assertIn(str(value).lower(), qs_args[key])

        # Also check that no query string args exist which are not expected
        for key in qs_args:
            self.assertIn(key, qs_args_expected)

    def test_list_params(self):
        ref_list = [self.new_ref()]
        filter_kwargs = {uuid.uuid4().hex: uuid.uuid4().hex}
        expected_path = self._get_expected_path()

        self.requests_mock.get(urlparse.urljoin(self.TEST_URL, expected_path),
                               json=self.encode(ref_list))

        self.manager.list(**filter_kwargs)
        self.assertQueryStringContains(**filter_kwargs)

    def test_find(self, ref=None):
        ref = ref or self.new_ref()
        ref_list = [ref]

        self.stub_entity('GET', entity=ref_list)

        returned = self.manager.find(name=getattr(ref, 'name', None))
        self.assertIsInstance(returned, self.model)
        for attr in ref:
            self.assertEqual(
                getattr(returned, attr),
                ref[attr],
                'Expected different %s' % attr)

        if hasattr(ref, 'name'):
            self.assertQueryStringIs('name=%s' % ref['name'])
        else:
            self.assertQueryStringIs('')

    def test_update(self, ref=None, req_ref=None):
        deprecations = self.useFixture(client_fixtures.Deprecations())
        deprecations.expect_deprecations()
        ref = ref or self.new_ref()

        self.stub_entity('PATCH', id=ref['id'], entity=ref)

        # req_ref argument allows you to specify a different
        # signature for the request when the manager does some
        # conversion before doing the request (e.g. converting
        # from datetime object to timestamp string)
        if req_ref:
            req_ref = req_ref.copy()
        else:
            req_ref = ref.copy()
            req_ref.pop('id')

        returned = self.manager.update(ref['id'], **parameterize(req_ref))
        self.assertIsInstance(returned, self.model)
        for attr in ref:
            self.assertEqual(
                getattr(returned, attr),
                ref[attr],
                'Expected different %s' % attr)
        self.assertEntityRequestBodyIs(req_ref)

    def test_delete(self, ref=None):
        ref = ref or self.new_ref()

        self.stub_entity('DELETE', id=ref['id'], status_code=204)
        self.manager.delete(ref['id'])


class TestRequestId(TestCase):
    resp = requests.Response()
    TEST_REQUEST_ID = uuid.uuid4().hex
    resp.headers['x-openstack-request-id'] = TEST_REQUEST_ID

    def setUp(self):
        super(TestRequestId, self).setUp()
        auth = v3.Token(auth_url='http://127.0.0.1:5000',
                        token=self.TEST_TOKEN)
        session_ = session.Session(auth=auth)
        self.client = client.Client(session=session_,
                                    include_metadata='True')._adapter



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from keystoneclient import base


class EndpointGroup(base.Resource):
    """Represents an identity endpoint group.

    Attributes:
        * id: a UUID that identifies the endpoint group
        * name: the endpoint group name
        * description: the endpoint group description
        * filters: representation of filters in the format of JSON that define
                   what endpoint entities are part of the group

    """

    pass


class EndpointGroupManager(base.CrudManager):
    """Manager class for Endpoint Groups."""

    resource_class = EndpointGroup
    collection_key = 'endpoint_groups'
    key = 'endpoint_group'
    base_url = 'OS-EP-FILTER'

    def create(self, name, filters, description=None, **kwargs):
        """Create an endpoint group.

        :param str name: the name of the endpoint group.
        :param str filters: representation of filters in the format of JSON
                            that define what endpoint entities are part of the
                            group.
        :param str description: a description of the endpoint group.
        :param kwargs: any other attribute provided will be passed to the
                       server.

        :returns: the created endpoint group returned from server.
        :rtype: :class:`keystoneclient.v3.endpoint_groups.EndpointGroup`

        """
        return super(EndpointGroupManager, self).create(
            name=name,
            filters=filters,
            description=description,
            **kwargs)

    def get(self, endpoint_group):
        """Retrieve an endpoint group.

        :param endpoint_group: the endpoint group to be retrieved from the
                               server.
        :type endpoint_group:
            str or :class:`keystoneclient.v3.endpoint_groups.EndpointGroup`

        :returns: the specified endpoint group returned from server.
        :rtype: :class:`keystoneclient.v3.endpoint_groups.EndpointGroup`

        """
        return super(EndpointGroupManager, self).get(
            endpoint_group_id=base.getid(endpoint_group))

    def check(self, endpoint_group):
        """Check if an endpoint group exists.

        :param endpoint_group: the endpoint group to be checked against the
                               server.
        :type endpoint_group:
            str or :class:`keystoneclient.v3.endpoint_groups.EndpointGroup`

        :returns: none if the specified endpoint group exists.

        """
        return super(EndpointGroupManager, self).head(
            endpoint_group_id=base.getid(endpoint_group))

    def list(self, **kwargs):
        """List endpoint groups.

        Any parameter provided will be passed to the server.

        :returns: a list of endpoint groups.
        :rtype: list of
                :class:`keystoneclient.v3.endpoint_groups.EndpointGroup`.

        """
        return super(EndpointGroupManager, self).list(**kwargs)

    def update(self, endpoint_group, name=None, filters=None,
               description=None, **kwargs):
        """Update an endpoint group.

        :param str name: the new name of the endpoint group.
        :param str filters: the new representation of filters in the format of
                            JSON that define what endpoint entities are part of
                            the group.
        :param str description: the new description of the endpoint group.
        :param kwargs: any other attribute provided will be passed to the
                       server.

        :returns: the updated endpoint group returned from server.
        :rtype: :class:`keystoneclient.v3.endpoint_groups.EndpointGroup`

        """
        return super(EndpointGroupManager, self).update(
            endpoint_group_id=base.getid(endpoint_group),
            name=name,
            filters=filters,
            description=description,
            **kwargs)

    def delete(self, endpoint_group):
        """Delete an endpoint group.

        :param endpoint_group: the endpoint group to be deleted on the server.
        :type endpoint_group:
            str or :class:`keystoneclient.v3.endpoint_groups.EndpointGroup`

        :returns: Response object with 204 status.
        :rtype: :class:`requests.models.Response`

        """
        return super(EndpointGroupManager, self).delete(
            endpoint_group_id=base.getid(endpoint_group))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import argparse
from unittest import mock
import uuid


from keystoneclient.auth.identity.generic import cli
from keystoneclient import exceptions
from keystoneclient.tests.unit import utils


class DefaultCliTests(utils.TestCase):

    def setUp(self):
        super(DefaultCliTests, self).setUp()
        self.deprecations.expect_deprecations()

    def new_plugin(self, argv):
        parser = argparse.ArgumentParser()
        cli.DefaultCLI.register_argparse_arguments(parser)
        opts = parser.parse_args(argv)
        return cli.DefaultCLI.load_from_argparse_arguments(opts)

    def test_endpoint_override(self):
        password = uuid.uuid4().hex
        url = uuid.uuid4().hex

        p = self.new_plugin(['--os-auth-url', 'url',
                             '--os-endpoint', url,
                             '--os-password', password])

        self.assertEqual(url, p.get_endpoint(None))
        self.assertEqual(password, p._password)

    def test_token_only_override(self):
        self.assertRaises(exceptions.CommandError,
                          self.new_plugin,
                          ['--os-token', uuid.uuid4().hex])

    def test_token_endpoint_override(self):
        token = uuid.uuid4().hex
        endpoint = uuid.uuid4().hex

        p = self.new_plugin(['--os-endpoint', endpoint,
                             '--os-token', token])

        self.assertEqual(endpoint, p.get_endpoint(None))
        self.assertEqual(token, p.get_token(None))

    def test_no_auth_url(self):
        exc = self.assertRaises(exceptions.CommandError,
                                self.new_plugin,
                                ['--os-username', uuid.uuid4().hex])

        self.assertIn('auth-url', str(exc))

    @mock.patch('sys.stdin', autospec=True)
    @mock.patch('getpass.getpass')
    def test_prompt_password(self, mock_getpass, mock_stdin):
        password = uuid.uuid4().hex

        mock_stdin.isatty = lambda: True
        mock_getpass.return_value = password

        p = self.new_plugin(['--os-auth-url', uuid.uuid4().hex,
                             '--os-username', uuid.uuid4().hex])

        self.assertEqual(password, p._password)

    @mock.patch('sys.stdin', autospec=True)
    @mock.patch('getpass.getpass')
    def test_prompt_no_password(self, mock_getpass, mock_stdin):
        mock_stdin.isatty = lambda: True
        mock_getpass.return_value = ''

        exc = self.assertRaises(exceptions.CommandError,
                                self.new_plugin,
                                ['--os-auth-url', uuid.uuid4().hex,
                                 '--os-username', uuid.uuid4().hex])

        self.assertIn('password', str(exc))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_config import cfg

from keystoneclient import access
from keystoneclient.auth.identity.v3 import federated


class OidcPassword(federated.FederatedBaseAuth):
    """Implement authentication plugin for OpenID Connect protocol.

    OIDC or OpenID Connect is a protocol for federated authentication.

    The OpenID Connect specification can be found at::
    ``http://openid.net/specs/openid-connect-core-1_0.html``
    """

    @classmethod
    def get_options(cls):
        options = super(OidcPassword, cls).get_options()
        options.extend([
            cfg.StrOpt('username', help='Username'),
            cfg.StrOpt('password', secret=True, help='Password'),
            cfg.StrOpt('client-id', help='OAuth 2.0 Client ID'),
            cfg.StrOpt('client-secret', secret=True,
                       help='OAuth 2.0 Client Secret'),
            cfg.StrOpt('access-token-endpoint',
                       help='OpenID Connect Provider Token Endpoint'),
            cfg.StrOpt('scope', default="profile",
                       help='OpenID Connect scope that is requested from OP')
        ])
        return options

    def __init__(self, auth_url, identity_provider, protocol,
                 username, password, client_id, client_secret,
                 access_token_endpoint, scope='profile',
                 grant_type='password'):
        """The OpenID Connect plugin.

        It expects the following:

        :param auth_url: URL of the Identity Service
        :type auth_url: string

        :param identity_provider: Name of the Identity Provider the client
                                  will authenticate against
        :type identity_provider: string

        :param protocol: Protocol name as configured in keystone
        :type protocol: string

        :param username: Username used to authenticate
        :type username: string

        :param password: Password used to authenticate
        :type password: string

        :param client_id: OAuth 2.0 Client ID
        :type client_id: string

        :param client_secret: OAuth 2.0 Client Secret
        :type client_secret: string

        :param access_token_endpoint: OpenID Connect Provider Token Endpoint,
                                      for example:
                                      https://localhost:8020/oidc/OP/token
        :type access_token_endpoint: string

        :param scope: OpenID Connect scope that is requested from OP,
                      defaults to "profile", for example: "profile email"
        :type scope: string

        :param grant_type: OpenID Connect grant type, it represents the flow
                           that is used to talk to the OP. Valid values are:
                           "authorization_code", "refresh_token", or
                           "password".
        :type grant_type: string
        """
        super(OidcPassword, self).__init__(auth_url, identity_provider,
                                           protocol)
        self._username = username
        self._password = password
        self.client_id = client_id
        self.client_secret = client_secret
        self.access_token_endpoint = access_token_endpoint
        self.scope = scope
        self.grant_type = grant_type

    @property
    def username(self):
        # Override to remove deprecation.
        return self._username

    @username.setter
    def username(self, value):
        # Override to remove deprecation.
        self._username = value

    @property
    def password(self):
        # Override to remove deprecation.
        return self._password

    @password.setter
    def password(self, value):
        # Override to remove deprecation.
        self._password = value

    def get_unscoped_auth_ref(self, session):
        """Authenticate with OpenID Connect and get back claims.

        This is a multi-step process. First an access token must be retrieved,
        to do this, the username and password, the OpenID Connect client ID
        and secret, and the access token endpoint must be known.

        Secondly, we then exchange the access token upon accessing the
        protected Keystone endpoint (federated auth URL). This will trigger
        the OpenID Connect Provider to perform a user introspection and
        retrieve information (specified in the scope) about the user in
        the form of an OpenID Connect Claim. These claims will be sent
        to Keystone in the form of environment variables.

        :param session: a session object to send out HTTP requests.
        :type session: keystoneclient.session.Session

        :returns: a token data representation
        :rtype: :py:class:`keystoneclient.access.AccessInfo`
        """
        # get an access token
        client_auth = (self.client_id, self.client_secret)
        payload = {'grant_type': self.grant_type, 'username': self.username,
                   'password': self.password, 'scope': self.scope}
        response = self._get_access_token(session, client_auth, payload,
                                          self.access_token_endpoint)
        access_token = response.json()['access_token']

        # use access token against protected URL
        headers = {'Authorization': 'Bearer ' + access_token}
        response = self._get_keystone_token(session, headers,
                                            self.federated_token_url)

        # grab the unscoped token
        token = response.headers['X-Subject-Token']
        token_json = response.json()['token']
        return access.AccessInfoV3(token, **token_json)

    def _get_access_token(self, session, client_auth, payload,
                          access_token_endpoint):
        """Exchange a variety of user supplied values for an access token.

        :param session: a session object to send out HTTP requests.
        :type session: keystoneclient.session.Session

        :param client_auth: a tuple representing client id and secret
        :type client_auth: tuple

        :param payload: a dict containing various OpenID Connect values, for
                        example::
                          {'grant_type': 'password', 'username': self.username,
                           'password': self.password, 'scope': self.scope}
        :type payload: dict

        :param access_token_endpoint: URL to use to get an access token, for
                                      example: https://localhost/oidc/token
        :type access_token_endpoint: string
        """
        op_response = session.post(self.access_token_endpoint,
                                   requests_auth=client_auth,
                                   data=payload,
                                   authenticated=False)
        return op_response

    def _get_keystone_token(self, session, headers, federated_token_url):
        r"""Exchange an acess token for a keystone token.

        By Sending the access token in an `Authorization: Bearer` header, to
        an OpenID Connect protected endpoint (Federated Token URL). The
        OpenID Connect server will use the access token to look up information
        about the authenticated user (this technique is called instrospection).
        The output of the instrospection will be an OpenID Connect Claim, that
        will be used against the mapping engine. Should the mapping engine
        succeed, a Keystone token will be presented to the user.

        :param session: a session object to send out HTTP requests.
        :type session: keystoneclient.session.Session

        :param headers: an Authorization header containing the access token.
        :type headers_: dict

        :param federated_auth_url: Protected URL for federated authentication,
                                   for example: https://localhost:5000/v3/\
                                   OS-FEDERATION/identity_providers/bluepages/\
                                   protocols/oidc/auth
        :type federated_auth_url: string
        """
        auth_response = session.post(self.federated_token_url,
                                     headers=headers,
                                     authenticated=False)
        return auth_response



# Copyright 2010 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
import urllib.parse as urlparse

from debtcollector import removals

from keystoneclient import exceptions
from keystoneclient import httpclient
from keystoneclient.i18n import _


_logger = logging.getLogger(__name__)


# NOTE(jamielennox): To be removed after Pike.
@removals.removed_class('keystoneclient.generic.client.Client',
                        message='Use keystoneauth discovery',
                        version='3.9.0',
                        removal_version='4.0.0')
class Client(httpclient.HTTPClient):
    """Client for the OpenStack Keystone pre-version calls API.

    :param string endpoint: A user-supplied endpoint URL for the keystone
                            service.
    :param integer timeout: Allows customization of the timeout for client
                            http requests. (optional)

    Example::

        >>> from keystoneclient.generic import client
        >>> root = client.Client(auth_url=KEYSTONE_URL)
        >>> versions = root.discover()
        ...
        >>> from keystoneclient.v2_0 import client as v2client
        >>> keystone = v2client.Client(auth_url=versions['v2.0']['url'])
        ...
        >>> user = keystone.users.get(USER_ID)
        >>> user.delete()

    """

    def __init__(self, endpoint=None, **kwargs):
        """Initialize a new client for the Keystone v2.0 API."""
        super(Client, self).__init__(endpoint=endpoint, **kwargs)
        self.endpoint = endpoint

    def discover(self, url=None):
        """Discover Keystone servers and return API versions supported.

        :param url: optional url to test (without version)

        Returns::

            {
                'message': 'Keystone found at http://127.0.0.1:5000/',
                'v2.0': {
                    'status': 'beta',
                    'url': 'http://127.0.0.1:5000/v2.0/',
                    'id': 'v2.0'
                },
            }

        """
        if url:
            return self._check_keystone_versions(url)
        else:
            return self._local_keystone_exists()

    def _local_keystone_exists(self):
        """Check if Keystone is available on default local port 35357."""
        results = self._check_keystone_versions("http://localhost:35357")
        if results is None:
            results = self._check_keystone_versions("https://localhost:35357")
        return results

    def _check_keystone_versions(self, url):
        """Call Keystone URL and detects the available API versions."""
        try:
            resp, body = self._request(url, "GET",
                                       headers={'Accept':
                                                'application/json'})
            # Multiple Choices status code is returned by the root
            # identity endpoint, with references to one or more
            # Identity API versions -- v3 spec
            # some cases we get No Content
            if resp.status_code in (200, 204, 300):
                try:
                    results = {}
                    if 'version' in body:
                        results['message'] = _("Keystone found at %s") % url
                        version = body['version']
                        # Stable/diablo incorrect format
                        id, status, version_url = (
                            self._get_version_info(version, url))
                        results[str(id)] = {"id": id,
                                            "status": status,
                                            "url": version_url}
                        return results
                    elif 'versions' in body:
                        # Correct format
                        results['message'] = _("Keystone found at %s") % url
                        for version in body['versions']['values']:
                            id, status, version_url = (
                                self._get_version_info(version, url))
                            results[str(id)] = {"id": id,
                                                "status": status,
                                                "url": version_url}
                        return results
                    else:
                        results['message'] = (
                            _("Unrecognized response from %s") % url)
                    return results
                except KeyError:
                    raise exceptions.AuthorizationFailure()
            elif resp.status_code == 305:
                return self._check_keystone_versions(resp['location'])
            else:
                raise exceptions.from_response(resp, "GET", url)
        except Exception:
            _logger.exception('Failed to detect available versions.')

    def discover_extensions(self, url=None):
        """Discover Keystone extensions supported.

        :param url: optional url to test (should have a version in it)

        Returns::

            {
                'message': 'Keystone extensions at http://127.0.0.1:35357/v2',
                'OS-KSEC2': 'OpenStack EC2 Credentials Extension',
            }

        """
        if url:
            return self._check_keystone_extensions(url)

    def _check_keystone_extensions(self, url):
        """Call Keystone URL and detects the available extensions."""
        try:
            if not url.endswith("/"):
                url += '/'
            resp, body = self._request("%sextensions" % url, "GET",
                                       headers={'Accept':
                                                'application/json'})
            if resp.status_code in (200, 204):  # some cases we get No Content
                if 'extensions' in body and 'values' in body['extensions']:
                    # Parse correct format (per contract)
                    extensions = body['extensions']['values']
                elif 'extensions' in body:
                    # Support incorrect, but prevalent format
                    extensions = body['extensions']
                else:
                    return dict(message=(
                        _('Unrecognized extensions response from %s') % url))

                return dict(self._get_extension_info(e) for e in extensions)
            elif resp.status_code == 305:
                return self._check_keystone_extensions(resp['location'])
            else:
                raise exceptions.from_response(
                    resp, "GET", "%sextensions" % url)
        except Exception:
            _logger.exception('Failed to check keystone extensions.')

    @staticmethod
    def _get_version_info(version, root_url):
        """Parse version information.

        :param version: a dict of a Keystone version response
        :param root_url: string url used to construct
            the version if no URL is provided.
        :returns: tuple - (verionId, versionStatus, versionUrl)
        """
        id = version['id']
        status = version['status']
        ref = urlparse.urljoin(root_url, id)
        if 'links' in version:
            for link in version['links']:
                if link['rel'] == 'self':
                    ref = link['href']
                    break
        return (id, status, ref)

    @staticmethod
    def _get_extension_info(extension):
        """Parse extension information.

        :param extension: a dict of a Keystone extension response
        :returns: tuple - (alias, name)
        """
        alias = extension['alias']
        name = extension['name']
        return (alias, name)



# Copyright 2014 IBM Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from keystoneclient.tests.unit.v3 import test_endpoint_filter
from keystoneclient.tests.unit.v3 import utils


class EndpointPolicyTests(utils.ClientTestCase,
                          test_endpoint_filter.EndpointTestUtils):
    """Test policy-endpoint associations (a.k.a. EndpointPolicy Extension)."""

    def setUp(self):
        super(EndpointPolicyTests, self).setUp()
        self.manager = self.client.endpoint_policy

    def new_policy_ref(self, **kwargs):
        kwargs.setdefault('id', uuid.uuid4().hex)
        kwargs.setdefault('type', uuid.uuid4().hex)
        kwargs.setdefault('blob', uuid.uuid4().hex)
        return kwargs

    def new_region_ref(self, **kwargs):
        kwargs = self.new_ref(**kwargs)
        return kwargs

    def new_service_ref(self, **kwargs):
        kwargs = self.new_ref(**kwargs)
        kwargs.setdefault('name', uuid.uuid4().hex)
        kwargs.setdefault('type', uuid.uuid4().hex)
        return kwargs

    def _crud_policy_association_for_endpoint_via_id(
            self, http_action, manager_action):
        policy_id = uuid.uuid4().hex
        endpoint_id = uuid.uuid4().hex

        self.stub_url(http_action,
                      ['policies', policy_id, self.manager.OS_EP_POLICY_EXT,
                       'endpoints', endpoint_id],
                      status_code=204)
        manager_action(policy=policy_id, endpoint=endpoint_id)

    def _crud_policy_association_for_endpoint_via_obj(
            self, http_action, manager_action):
        policy_ref = self.new_policy_ref()
        endpoint_ref = self.new_endpoint_ref()
        policy = self.client.policies.resource_class(
            self.client.policies, policy_ref, loaded=True)
        endpoint = self.client.endpoints.resource_class(
            self.client.endpoints, endpoint_ref, loaded=True)

        self.stub_url(http_action,
                      ['policies', policy_ref['id'],
                       self.manager.OS_EP_POLICY_EXT,
                       'endpoints', endpoint_ref['id']],
                      status_code=204)
        manager_action(policy=policy, endpoint=endpoint)

    def test_create_policy_association_for_endpoint_via_id(self):
        self._crud_policy_association_for_endpoint_via_id(
            'PUT', self.manager.create_policy_association_for_endpoint)

    def test_create_policy_association_for_endpoint_via_obj(self):
        self._crud_policy_association_for_endpoint_via_obj(
            'PUT', self.manager.create_policy_association_for_endpoint)

    def test_check_policy_association_for_endpoint_via_id(self):
        self._crud_policy_association_for_endpoint_via_id(
            'HEAD', self.manager.check_policy_association_for_endpoint)

    def test_check_policy_association_for_endpoint_via_obj(self):
        self._crud_policy_association_for_endpoint_via_obj(
            'HEAD', self.manager.check_policy_association_for_endpoint)

    def test_delete_policy_association_for_endpoint_via_id(self):
        self._crud_policy_association_for_endpoint_via_id(
            'DELETE', self.manager.delete_policy_association_for_endpoint)

    def test_delete_policy_association_for_endpoint_via_obj(self):
        self._crud_policy_association_for_endpoint_via_obj(
            'DELETE', self.manager.delete_policy_association_for_endpoint)

    def _crud_policy_association_for_service_via_id(
            self, http_action, manager_action):
        policy_id = uuid.uuid4().hex
        service_id = uuid.uuid4().hex

        self.stub_url(http_action,
                      ['policies', policy_id, self.manager.OS_EP_POLICY_EXT,
                       'services', service_id],
                      status_code=204)
        manager_action(policy=policy_id, service=service_id)

    def _crud_policy_association_for_service_via_obj(
            self, http_action, manager_action):
        policy_ref = self.new_policy_ref()
        service_ref = self.new_service_ref()
        policy = self.client.policies.resource_class(
            self.client.policies, policy_ref, loaded=True)
        service = self.client.services.resource_class(
            self.client.services, service_ref, loaded=True)

        self.stub_url(http_action,
                      ['policies', policy_ref['id'],
                       self.manager.OS_EP_POLICY_EXT,
                       'services', service_ref['id']],
                      status_code=204)
        manager_action(policy=policy, service=service)

    def test_create_policy_association_for_service_via_id(self):
        self._crud_policy_association_for_service_via_id(
            'PUT', self.manager.create_policy_association_for_service)

    def test_create_policy_association_for_service_via_obj(self):
        self._crud_policy_association_for_service_via_obj(
            'PUT', self.manager.create_policy_association_for_service)

    def test_check_policy_association_for_service_via_id(self):
        self._crud_policy_association_for_service_via_id(
            'HEAD', self.manager.check_policy_association_for_service)

    def test_check_policy_association_for_service_via_obj(self):
        self._crud_policy_association_for_service_via_obj(
            'HEAD', self.manager.check_policy_association_for_service)

    def test_delete_policy_association_for_service_via_id(self):
        self._crud_policy_association_for_service_via_id(
            'DELETE', self.manager.delete_policy_association_for_service)

    def test_delete_policy_association_for_service_via_obj(self):
        self._crud_policy_association_for_service_via_obj(
            'DELETE', self.manager.delete_policy_association_for_service)

    def _crud_policy_association_for_region_and_service_via_id(
            self, http_action, manager_action):
        policy_id = uuid.uuid4().hex
        region_id = uuid.uuid4().hex
        service_id = uuid.uuid4().hex

        self.stub_url(http_action,
                      ['policies', policy_id, self.manager.OS_EP_POLICY_EXT,
                       'services', service_id, 'regions', region_id],
                      status_code=204)
        manager_action(policy=policy_id, region=region_id, service=service_id)

    def _crud_policy_association_for_region_and_service_via_obj(
            self, http_action, manager_action):
        policy_ref = self.new_policy_ref()
        region_ref = self.new_region_ref()
        service_ref = self.new_service_ref()
        policy = self.client.policies.resource_class(
            self.client.policies, policy_ref, loaded=True)
        region = self.client.regions.resource_class(
            self.client.regions, region_ref, loaded=True)
        service = self.client.services.resource_class(
            self.client.services, service_ref, loaded=True)

        self.stub_url(http_action,
                      ['policies', policy_ref['id'],
                       self.manager.OS_EP_POLICY_EXT,
                       'services', service_ref['id'],
                       'regions', region_ref['id']],
                      status_code=204)
        manager_action(policy=policy, region=region, service=service)

    def test_create_policy_association_for_region_and_service_via_id(self):
        self._crud_policy_association_for_region_and_service_via_id(
            'PUT',
            self.manager.create_policy_association_for_region_and_service)

    def test_create_policy_association_for_region_and_service_via_obj(self):
        self._crud_policy_association_for_region_and_service_via_obj(
            'PUT',
            self.manager.create_policy_association_for_region_and_service)

    def test_check_policy_association_for_region_and_service_via_id(self):
        self._crud_policy_association_for_region_and_service_via_id(
            'HEAD',
            self.manager.check_policy_association_for_region_and_service)

    def test_check_policy_association_for_region_and_service_via_obj(self):
        self._crud_policy_association_for_region_and_service_via_obj(
            'HEAD',
            self.manager.check_policy_association_for_region_and_service)

    def test_delete_policy_association_for_region_and_service_via_id(self):
        self._crud_policy_association_for_region_and_service_via_id(
            'DELETE',
            self.manager.delete_policy_association_for_region_and_service)

    def test_delete_policy_association_for_region_and_service_via_obj(self):
        self._crud_policy_association_for_region_and_service_via_obj(
            'DELETE',
            self.manager.delete_policy_association_for_region_and_service)

    def test_get_policy_for_endpoint(self):
        endpoint_id = uuid.uuid4().hex
        expected_policy = self.new_policy_ref()

        self.stub_url('GET',
                      ['endpoints', endpoint_id, self.manager.OS_EP_POLICY_EXT,
                       'policy'],
                      json={'policy': expected_policy},
                      status_code=200)

        policy_resp = self.manager.get_policy_for_endpoint(
            endpoint=endpoint_id)

        self.assertEqual(expected_policy['id'], policy_resp.id)
        self.assertEqual(expected_policy['blob'], policy_resp.blob)
        self.assertEqual(expected_policy['type'], policy_resp.type)

    def test_list_endpoints_for_policy(self):
        policy_id = uuid.uuid4().hex
        endpoints = {'endpoints': [self.new_endpoint_ref(),
                                   self.new_endpoint_ref()]}
        self.stub_url('GET',
                      ['policies', policy_id, self.manager.OS_EP_POLICY_EXT,
                       'endpoints'],
                      json=endpoints,
                      status_code=200)

        endpoints_resp = self.manager.list_endpoints_for_policy(
            policy=policy_id)

        expected_endpoint_ids = [
            endpoint['id'] for endpoint in endpoints['endpoints']]
        actual_endpoint_ids = [endpoint.id for endpoint in endpoints_resp]
        self.assertEqual(expected_endpoint_ids, actual_endpoint_ids)



# Copyright 2011 OpenStack Foundation
# Copyright 2011 Nebula, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from debtcollector import removals

from keystoneclient import base
from keystoneclient import exceptions
from keystoneclient.i18n import _


class Role(base.Resource):
    """Represents an Identity role.

    Attributes:
        * id: a uuid that identifies the role
        * name: user-facing identifier
        * domain: optional domain for the role

    """

    pass


class InferenceRule(base.Resource):
    """Represents a rule that states one role implies another.

    Attributes:
        * prior_role: this role implies the other
        * implied_role: this role is implied by the other

    """

    pass


class RoleManager(base.CrudManager):
    """Manager class for manipulating Identity roles."""

    resource_class = Role
    collection_key = 'roles'
    key = 'role'
    deprecation_msg = 'keystoneclient.v3.roles.InferenceRuleManager'

    def _role_grants_base_url(self, user, group, system, domain, project,
                              use_inherit_extension):
        # When called, we have already checked that only one of user & group
        # and one of domain & project have been specified
        params = {}

        if project:
            params['project_id'] = base.getid(project)
            base_url = '/projects/%(project_id)s'
        elif domain:
            params['domain_id'] = base.getid(domain)
            base_url = '/domains/%(domain_id)s'
        elif system:
            if system == 'all':
                base_url = '/system'
            else:
                # NOTE(lbragstad): If we've made it this far, a user is
                # attempting to do something with system scope that isn't
                # supported yet (e.g. 'all' is currently the only supported
                # system scope). In the future that may change but until then
                # we should fail like we would if a user provided a bogus
                # project name or domain ID.
                msg = _("Only a system scope of 'all' is currently supported")
                raise exceptions.ValidationError(msg)

        if use_inherit_extension:
            base_url = '/OS-INHERIT' + base_url

        if user:
            params['user_id'] = base.getid(user)
            base_url += '/users/%(user_id)s'
        elif group:
            params['group_id'] = base.getid(group)
            base_url += '/groups/%(group_id)s'

        return base_url % params

    def _enforce_mutually_exclusive_group(self, system, domain, project):
        if not system:
            if domain and project:
                msg = _('Specify either a domain or project, not both')
                raise exceptions.ValidationError(msg)
            elif not (domain or project):
                msg = _('Must specify either system, domain, or project')
                raise exceptions.ValidationError(msg)
        elif system:
            if domain and project:
                msg = _(
                    'Specify either system, domain, or project, not all three.'
                )
                raise exceptions.ValidationError(msg)
            if domain:
                msg = _('Specify either system or a domain, not both')
                raise exceptions.ValidationError(msg)
            if project:
                msg = _('Specify either a system or project, not both')
                raise exceptions.ValidationError(msg)

    def _require_user_xor_group(self, user, group):
        if user and group:
            msg = _('Specify either a user or group, not both')
            raise exceptions.ValidationError(msg)
        elif not (user or group):
            msg = _('Must specify either a user or group')
            raise exceptions.ValidationError(msg)

    def create(self, name, domain=None, **kwargs):
        """Create a role.

        :param str name: the name of the role.
        :param domain: the domain of the role. If a value is passed it is a
                       domain-scoped role, otherwise it's a global role.
        :type domain: str or :class:`keystoneclient.v3.domains.Domain`
        :param kwargs: any other attribute provided will be passed to the
                       server.

        :returns: the created role returned from server.
        :rtype: :class:`keystoneclient.v3.roles.Role`

        """
        domain_id = None
        if domain:
            domain_id = base.getid(domain)

        return super(RoleManager, self).create(
            name=name,
            domain_id=domain_id,
            **kwargs)

    def get(self, role):
        """Retrieve a role.

        :param role: the role to be retrieved from the server.
        :type role: str or :class:`keystoneclient.v3.roles.Role`

        :returns: the specified role returned from server.
        :rtype: :class:`keystoneclient.v3.roles.Role`

        """
        return super(RoleManager, self).get(role_id=base.getid(role))

    def list(self, user=None, group=None, system=None, domain=None,
             project=None, os_inherit_extension_inherited=False, **kwargs):
        """List roles and role grants.

        :param user: filter in role grants for the specified user on a
                     resource. Domain or project must be specified.
                     User and group are mutually exclusive.
        :type user: str or :class:`keystoneclient.v3.users.User`
        :param group: filter in role grants for the specified group on a
                      resource. Domain or project must be specified.
                      User and group are mutually exclusive.
        :type group: str or :class:`keystoneclient.v3.groups.Group`
        :param domain: filter in role grants on the specified domain. Either
                       user or group must be specified. Project, domain, and
                       system are mutually exclusive.
        :type domain: str or :class:`keystoneclient.v3.domains.Domain`
        :param project: filter in role grants on the specified project. Either
                       user or group must be specified. Project, domain and
                       system are mutually exclusive.
        :type project: str or :class:`keystoneclient.v3.projects.Project`
        :param bool os_inherit_extension_inherited: OS-INHERIT will be used.
                                                    It provides the ability for
                                                    projects to inherit role
                                                    assignments from their
                                                    domains or from parent
                                                    projects in the hierarchy.
        :param kwargs: any other attribute provided will filter roles on.

        :returns: a list of roles.
        :rtype: list of :class:`keystoneclient.v3.roles.Role`

        """
        if os_inherit_extension_inherited:
            kwargs['tail'] = '/inherited_to_projects'
        if user or group:
            self._require_user_xor_group(user, group)
            self._enforce_mutually_exclusive_group(system, domain, project)

            base_url = self._role_grants_base_url(
                user, group, system, domain, project,
                os_inherit_extension_inherited
            )
            return super(RoleManager, self).list(base_url=base_url,
                                                 **kwargs)

        return super(RoleManager, self).list(**kwargs)

    def update(self, role, name=None, **kwargs):
        """Update a role.

        :param role: the role to be updated on the server.
        :type role: str or :class:`keystoneclient.v3.roles.Role`
        :param str name: the new name of the role.
        :param kwargs: any other attribute provided will be passed to server.

        :returns: the updated role returned from server.
        :rtype: :class:`keystoneclient.v3.roles.Role`

        """
        return super(RoleManager, self).update(
            role_id=base.getid(role),
            name=name,
            **kwargs)

    def delete(self, role):
        """Delete a role.

        When a role is deleted all the role inferences that have deleted role
        as prior role will be deleted as well.

        :param role: the role to be deleted on the server.
        :type role: str or :class:`keystoneclient.v3.roles.Role`

        :returns: Response object with 204 status.
        :rtype: :class:`requests.models.Response`

        """
        return super(RoleManager, self).delete(
            role_id=base.getid(role))

    def grant(self, role, user=None, group=None, system=None, domain=None,
              project=None, os_inherit_extension_inherited=False, **kwargs):
        """Grant a role to a user or group on a domain or project.

        :param role: the role to be granted on the server.
        :type role: str or :class:`keystoneclient.v3.roles.Role`
        :param user: the specified user to have the role granted on a resource.
                     Domain or project must be specified. User and group are
                     mutually exclusive.
        :type user: str or :class:`keystoneclient.v3.users.User`
        :param group: the specified group to have the role granted on a
                      resource. Domain or project must be specified.
                      User and group are mutually exclusive.
        :type group: str or :class:`keystoneclient.v3.groups.Group`
        :param system: system information to grant the role on. Project,
                       domain, and system are mutually exclusive.
        :type system: str
        :param domain: the domain in which the role will be granted. Either
                       user or group must be specified. Project, domain, and
                       system are mutually exclusive.
        :type domain: str or :class:`keystoneclient.v3.domains.Domain`
        :param project: the project in which the role will be granted. Either
                       user or group must be specified. Project, domain, and
                       system are mutually exclusive.
        :type project: str or :class:`keystoneclient.v3.projects.Project`
        :param bool os_inherit_extension_inherited: OS-INHERIT will be used.
                                                    It provides the ability for
                                                    projects to inherit role
                                                    assignments from their
                                                    domains or from parent
                                                    projects in the hierarchy.
        :param kwargs: any other attribute provided will be passed to server.

        :returns: the granted role returned from server.
        :rtype: :class:`keystoneclient.v3.roles.Role`

        """
        self._enforce_mutually_exclusive_group(system, domain, project)
        self._require_user_xor_group(user, group)

        if os_inherit_extension_inherited:
            kwargs['tail'] = '/inherited_to_projects'

        base_url = self._role_grants_base_url(
            user, group, system, domain, project,
            os_inherit_extension_inherited)
        return super(RoleManager, self).put(base_url=base_url,
                                            role_id=base.getid(role),
                                            **kwargs)

    def check(self, role, user=None, group=None, system=None, domain=None,
              project=None, os_inherit_extension_inherited=False, **kwargs):
        """Check if a user or group has a role on a domain or project.

        :param user: check for role grants for the specified user on a
                     resource. Domain or project must be specified.
                     User and group are mutually exclusive.
        :type user: str or :class:`keystoneclient.v3.users.User`
        :param group: check for role grants for the specified group on a
                      resource. Domain or project must be specified.
                      User and group are mutually exclusive.
        :type group: str or :class:`keystoneclient.v3.groups.Group`
        :param system: check for role  grants on the system. Project, domain,
                       and system are mutually exclusive.
        :type system: str
        :param domain: check for role grants on the specified domain. Either
                       user or group must be specified. Project, domain, and
                       system are mutually exclusive.
        :type domain: str or :class:`keystoneclient.v3.domains.Domain`
        :param project: check for role grants on the specified project. Either
                       user or group must be specified. Project, domain, and
                       system are mutually exclusive.
        :type project: str or :class:`keystoneclient.v3.projects.Project`
        :param bool os_inherit_extension_inherited: OS-INHERIT will be used.
                                                    It provides the ability for
                                                    projects to inherit role
                                                    assignments from their
                                                    domains or from parent
                                                    projects in the hierarchy.
        :param kwargs: any other attribute provided will be passed to server.

        :returns: the specified role returned from server if it exists.
        :rtype: :class:`keystoneclient.v3.roles.Role`

        :returns: Response object with 204 status if specified role
                  doesn't exist.
        :rtype: :class:`requests.models.Response`

        """
        self._enforce_mutually_exclusive_group(system, domain, project)
        self._require_user_xor_group(user, group)

        if os_inherit_extension_inherited:
            kwargs['tail'] = '/inherited_to_projects'

        base_url = self._role_grants_base_url(
            user, group, system, domain, project,
            os_inherit_extension_inherited)
        return super(RoleManager, self).head(
            base_url=base_url,
            role_id=base.getid(role),
            os_inherit_extension_inherited=os_inherit_extension_inherited,
            **kwargs)

    def revoke(self, role, user=None, group=None, system=None, domain=None,
               project=None, os_inherit_extension_inherited=False, **kwargs):
        """Revoke a role from a user or group on a domain or project.

        :param user: revoke role grants for the specified user on a
                     resource. Domain or project must be specified.
                     User and group are mutually exclusive.
        :type user: str or :class:`keystoneclient.v3.users.User`
        :param group: revoke role grants for the specified group on a
                      resource. Domain or project must be specified.
                      User and group are mutually exclusive.
        :type group: str or :class:`keystoneclient.v3.groups.Group`
        :param system: revoke role grants on the system. Project, domain, and
                       system are mutually exclusive.
        :type system: str
        :param domain: revoke role grants on the specified domain. Either
                       user or group must be specified. Project, domain, and
                       system are mutually exclusive.
        :type domain: str or :class:`keystoneclient.v3.domains.Domain`
        :param project: revoke role grants on the specified project. Either
                       user or group must be specified. Project, domain, and
                       system are mutually exclusive.
        :type project: str or :class:`keystoneclient.v3.projects.Project`
        :param bool os_inherit_extension_inherited: OS-INHERIT will be used.
                                                    It provides the ability for
                                                    projects to inherit role
                                                    assignments from their
                                                    domains or from parent
                                                    projects in the hierarchy.
        :param kwargs: any other attribute provided will be passed to server.

        :returns: the revoked role returned from server.
        :rtype: list of :class:`keystoneclient.v3.roles.Role`

        """
        self._enforce_mutually_exclusive_group(system, domain, project)
        self._require_user_xor_group(user, group)

        if os_inherit_extension_inherited:
            kwargs['tail'] = '/inherited_to_projects'

        base_url = self._role_grants_base_url(
            user, group, system, domain, project,
            os_inherit_extension_inherited)
        return super(RoleManager, self).delete(
            base_url=base_url,
            role_id=base.getid(role),
            os_inherit_extension_inherited=os_inherit_extension_inherited,
            **kwargs)

    @removals.remove(message='Use %s.create instead.' % deprecation_msg,
                     version='3.9.0', removal_version='4.0.0')
    def create_implied(self, prior_role, implied_role, **kwargs):
        return InferenceRuleManager(self.client).create(prior_role,
                                                        implied_role)

    @removals.remove(message='Use %s.delete instead.' % deprecation_msg,
                     version='3.9.0', removal_version='4.0.0')
    def delete_implied(self, prior_role, implied_role, **kwargs):
        return InferenceRuleManager(self.client).delete(prior_role,
                                                        implied_role)

    @removals.remove(message='Use %s.get instead.' % deprecation_msg,
                     version='3.9.0', removal_version='4.0.0')
    def get_implied(self, prior_role, implied_role, **kwargs):
        return InferenceRuleManager(self.client).get(prior_role,
                                                     implied_role)

    @removals.remove(message='Use %s.check instead.' % deprecation_msg,
                     version='3.9.0', removal_version='4.0.0')
    def check_implied(self, prior_role, implied_role, **kwargs):
        return InferenceRuleManager(self.client).check(prior_role,
                                                       implied_role)

    @removals.remove(message='Use %s.list_inference_roles' % deprecation_msg,
                     version='3.9.0', removal_version='4.0.0')
    def list_role_inferences(self, **kwargs):
        return InferenceRuleManager(self.client).list_inference_roles()


class InferenceRuleManager(base.CrudManager):
    """Manager class for manipulating Identity inference rules."""

    resource_class = InferenceRule
    collection_key = 'role_inferences'
    key = 'role_inference'

    def _implied_role_url_tail(self, prior_role, implied_role):
        base_url = ('/%(prior_role_id)s/implies/%(implied_role_id)s' %
                    {'prior_role_id': base.getid(prior_role),
                     'implied_role_id': base.getid(implied_role)})
        return base_url

    def create(self, prior_role, implied_role):
        """Create an inference rule.

        An inference rule is comprised of two roles, a prior role and an
        implied role. The prior role will imply the implied role.

        Valid HTTP return codes:

            * 201: Resource is created successfully
            * 404: A role cannot be found
            * 409: The inference rule already exists

        :param prior_role: the role which implies ``implied_role``.
        :type role: str or :class:`keystoneclient.v3.roles.Role`
        :param implied_role: the role which is implied by ``prior_role``.
        :type role: str or :class:`keystoneclient.v3.roles.Role`

        :returns: a newly created role inference returned from server.
        :rtype: :class:`keystoneclient.v3.roles.InferenceRule`

        """
        url_tail = self._implied_role_url_tail(prior_role, implied_role)
        _resp, body = self.client.put("/roles" + url_tail)
        return self._prepare_return_value(
            _resp, self.resource_class(self, body['role_inference']))

    def delete(self, prior_role, implied_role):
        """Delete an inference rule.

        When deleting an inference rule, both roles are required. Note that
        neither role is deleted, only the inference relationship is dissolved.

        Valid HTTP return codes:

            * 204: Delete request is accepted
            * 404: A role cannot be found

        :param prior_role: the role which implies ``implied_role``.
        :type role: str or :class:`keystoneclient.v3.roles.Role`
        :param implied_role: the role which is implied by ``prior_role``.
        :type role: str or :class:`keystoneclient.v3.roles.Role`

        :returns: Response object with 204 status.
        :rtype: :class:`requests.models.Response`

        """
        url_tail = self._implied_role_url_tail(prior_role, implied_role)
        return self._delete("/roles" + url_tail)

    def get(self, prior_role, implied_role):
        """Retrieve an inference rule.

        Valid HTTP return codes:

            * 200: Inference rule is returned
            * 404: A role cannot be found

        :param prior_role: the role which implies ``implied_role``.
        :type role: str or :class:`keystoneclient.v3.roles.Role`
        :param implied_role: the role which is implied by ``prior_role``.
        :type role: str or :class:`keystoneclient.v3.roles.Role`

        :returns: the specified role inference returned from server.
        :rtype: :class:`keystoneclient.v3.roles.InferenceRule`

        """
        url_tail = self._implied_role_url_tail(prior_role, implied_role)
        _resp, body = self.client.get("/roles" + url_tail)
        return self._prepare_return_value(
            _resp, self.resource_class(self, body['role_inference']))

    def list(self, prior_role):
        """List all roles that a role may imply.

        Valid HTTP return codes:

            * 200: List of inference rules are returned
            * 404: A role cannot be found

        :param prior_role: the role which implies ``implied_role``.
        :type role: str or :class:`keystoneclient.v3.roles.Role`

        :returns: the specified role inference returned from server.
        :rtype: :class:`keystoneclient.v3.roles.InferenceRule`

        """
        url_tail = ('/%s/implies' % base.getid(prior_role))
        _resp, body = self.client.get("/roles" + url_tail)
        return self._prepare_return_value(
            _resp, self.resource_class(self, body['role_inference']))

    def check(self, prior_role, implied_role):
        """Check if an inference rule exists.

        Valid HTTP return codes:

            * 204: The rule inference exists
            * 404: A role cannot be found

        :param prior_role: the role which implies ``implied_role``.
        :type role: str or :class:`keystoneclient.v3.roles.Role`
        :param implied_role: the role which is implied by ``prior_role``.
        :type role: str or :class:`keystoneclient.v3.roles.Role`

        :returns: response object with 204 status returned from server.
        :rtype: :class:`requests.models.Response`

        """
        url_tail = self._implied_role_url_tail(prior_role, implied_role)
        return self._head("/roles" + url_tail)

    def list_inference_roles(self):
        """List all rule inferences.

        Valid HTTP return codes:

            * 200: All inference rules are returned

        :param kwargs: attributes provided will be passed to the server.

        :returns: a list of inference rules.
        :rtype: list of :class:`keystoneclient.v3.roles.InferenceRule`

        """
        return super(InferenceRuleManager, self).list()

    def update(self, **kwargs):
        raise exceptions.MethodNotImplemented(
            _('Update not supported for rule inferences'))

    def find(self, **kwargs):
        raise exceptions.MethodNotImplemented(
            _('Find not supported for rule inferences'))

    def put(self, **kwargs):
        raise exceptions.MethodNotImplemented(
            _('Put not supported for rule inferences'))



# Copyright 2014 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from keystoneclient import base
from keystoneclient.i18n import _
from keystoneclient.v3 import endpoints
from keystoneclient.v3 import policies


class EndpointPolicyManager(base.Manager):
    """Manager class for manipulating endpoint-policy associations."""

    OS_EP_POLICY_EXT = 'OS-ENDPOINT-POLICY'

    def _act_on_policy_association_for_endpoint(
            self, policy, endpoint, action):
        if not (policy and endpoint):
            raise ValueError(_('policy and endpoint are required'))

        policy_id = base.getid(policy)
        endpoint_id = base.getid(endpoint)
        url = ('/policies/%(policy_id)s/%(ext_name)s'
               '/endpoints/%(endpoint_id)s') % {
                   'policy_id': policy_id,
                   'ext_name': self.OS_EP_POLICY_EXT,
                   'endpoint_id': endpoint_id}
        return action(url=url)

    def create_policy_association_for_endpoint(self, policy, endpoint):
        """Create an association between a policy and an endpoint."""
        return self._act_on_policy_association_for_endpoint(
            policy, endpoint, self._put)

    def check_policy_association_for_endpoint(self, policy, endpoint):
        """Check an association between a policy and an endpoint."""
        return self._act_on_policy_association_for_endpoint(
            policy, endpoint, self._head)

    def delete_policy_association_for_endpoint(self, policy, endpoint):
        """Delete an association between a policy and an endpoint."""
        return self._act_on_policy_association_for_endpoint(
            policy, endpoint, self._delete)

    def _act_on_policy_association_for_service(self, policy, service, action):
        if not (policy and service):
            raise ValueError(_('policy and service are required'))

        policy_id = base.getid(policy)
        service_id = base.getid(service)
        url = ('/policies/%(policy_id)s/%(ext_name)s'
               '/services/%(service_id)s') % {
                   'policy_id': policy_id,
                   'ext_name': self.OS_EP_POLICY_EXT,
                   'service_id': service_id}
        return action(url=url)

    def create_policy_association_for_service(self, policy, service):
        """Create an association between a policy and a service."""
        return self._act_on_policy_association_for_service(
            policy, service, self._put)

    def check_policy_association_for_service(self, policy, service):
        """Check an association between a policy and a service."""
        return self._act_on_policy_association_for_service(
            policy, service, self._head)

    def delete_policy_association_for_service(self, policy, service):
        """Delete an association between a policy and a service."""
        return self._act_on_policy_association_for_service(
            policy, service, self._delete)

    def _act_on_policy_association_for_region_and_service(
            self, policy, region, service, action):
        if not (policy and region and service):
            raise ValueError(_('policy, region and service are required'))

        policy_id = base.getid(policy)
        region_id = base.getid(region)
        service_id = base.getid(service)
        url = ('/policies/%(policy_id)s/%(ext_name)s'
               '/services/%(service_id)s/regions/%(region_id)s') % {
                   'policy_id': policy_id,
                   'ext_name': self.OS_EP_POLICY_EXT,
                   'service_id': service_id,
                   'region_id': region_id}
        return action(url=url)

    def create_policy_association_for_region_and_service(
            self, policy, region, service):
        """Create an association between a policy and a service in a region."""
        return self._act_on_policy_association_for_region_and_service(
            policy, region, service, self._put)

    def check_policy_association_for_region_and_service(
            self, policy, region, service):
        """Check an association between a policy and a service in a region."""
        return self._act_on_policy_association_for_region_and_service(
            policy, region, service, self._head)

    def delete_policy_association_for_region_and_service(
            self, policy, region, service):
        """Delete an association between a policy and a service in a region."""
        return self._act_on_policy_association_for_region_and_service(
            policy, region, service, self._delete)

    def get_policy_for_endpoint(self, endpoint):
        """Get the effective policy for an endpoint.

        :param endpoint: endpoint object or ID

        :returns: policies.Policy object

        """
        if not endpoint:
            raise ValueError(_('endpoint is required'))

        endpoint_id = base.getid(endpoint)
        url = ('/endpoints/%(endpoint_id)s/%(ext_name)s/policy') % {
            'endpoint_id': endpoint_id,
            'ext_name': self.OS_EP_POLICY_EXT}

        resp, body = self.client.get(url)
        return self._prepare_return_value(
            resp, policies.Policy(self, body[policies.PolicyManager.key],
                                  loaded=True))

    def list_endpoints_for_policy(self, policy):
        """List endpoints with the effective association to a policy.

        :param policy: policy object or ID

        :returns: list of endpoints that are associated with the policy

        """
        if not policy:
            raise ValueError(_('policy is required'))

        policy_id = base.getid(policy)
        url = ('/policies/%(policy_id)s/%(ext_name)s/endpoints') % {
            'policy_id': policy_id,
            'ext_name': self.OS_EP_POLICY_EXT}
        return self._list(
            url,
            endpoints.EndpointManager.collection_key,
            obj_class=endpoints.EndpointManager.resource_class)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from keystoneclient import base
from keystoneclient.v3.contrib.oauth1 import utils


class Consumer(base.Resource):
    """Represents an OAuth consumer.

    Attributes:
        * id: a uuid that identifies the consumer
        * description: a short description of the consumer
    """

    pass


class ConsumerManager(base.CrudManager):
    """Manager class for manipulating identity consumers."""

    resource_class = Consumer
    collection_key = 'consumers'
    key = 'consumer'
    base_url = utils.OAUTH_PATH

    def create(self, description=None, **kwargs):
        return super(ConsumerManager, self).create(
            description=description,
            **kwargs)

    def get(self, consumer):
        return super(ConsumerManager, self).get(
            consumer_id=base.getid(consumer))

    def update(self, consumer, description=None, **kwargs):
        return super(ConsumerManager, self).update(
            consumer_id=base.getid(consumer),
            description=description,
            **kwargs)

    def delete(self, consumer):
        return super(ConsumerManager, self).delete(
            consumer_id=base.getid(consumer))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import uuid

from keystoneauth1 import exceptions
from keystoneauth1 import fixture

from keystoneclient.tests.unit.v2_0 import utils
from keystoneclient.v2_0 import client
from keystoneclient.v2_0 import tenants
from keystoneclient.v2_0 import users


class TenantTests(utils.ClientTestCase):
    def setUp(self):
        super(TenantTests, self).setUp()

        self.INVIS_ID = uuid.uuid4().hex
        self.DEMO_ID = uuid.uuid4().hex
        self.ADMIN_ID = uuid.uuid4().hex
        self.EXTRAS_ID = uuid.uuid4().hex

        self.TEST_TENANTS = {
            "tenants": {
                "values": [
                    {
                        "enabled": True,
                        "description": "A description change!",
                        "name": "invisible_to_admin",
                        "id": self.INVIS_ID,
                    },
                    {
                        "enabled": True,
                        "description": "None",
                        "name": "demo",
                        "id": self.DEMO_ID,
                    },
                    {
                        "enabled": True,
                        "description": "None",
                        "name": "admin",
                        "id": self.ADMIN_ID,
                    },
                    {
                        "extravalue01": "metadata01",
                        "enabled": True,
                        "description": "For testing extras",
                        "name": "test_extras",
                        "id": self.EXTRAS_ID,
                    }
                ],
                "links": [],
            },
        }

    def test_create(self):
        req_body = {
            "tenant": {
                "name": "tenantX",
                "description": "Like tenant 9, but better.",
                "enabled": True,
                "extravalue01": "metadata01",
            },
        }
        id_ = uuid.uuid4().hex
        resp_body = {
            "tenant": {
                "name": "tenantX",
                "enabled": True,
                "id": id_,
                "description": "Like tenant 9, but better.",
                "extravalue01": "metadata01",
            }
        }
        self.stub_url('POST', ['tenants'], json=resp_body)

        tenant = self.client.tenants.create(
            req_body['tenant']['name'],
            req_body['tenant']['description'],
            req_body['tenant']['enabled'],
            extravalue01=req_body['tenant']['extravalue01'],
            name="don't overwrite priors")
        self.assertIsInstance(tenant, tenants.Tenant)
        self.assertEqual(tenant.id, id_)
        self.assertEqual(tenant.name, "tenantX")
        self.assertEqual(tenant.description, "Like tenant 9, but better.")
        self.assertEqual(tenant.extravalue01, "metadata01")
        self.assertRequestBodyIs(json=req_body)

    def test_duplicate_create(self):
        req_body = {
            "tenant": {
                "name": "tenantX",
                "description": "The duplicate tenant.",
                "enabled": True
            },
        }
        resp_body = {
            "error": {
                "message": "Conflict occurred attempting to store project.",
                "code": 409,
                "title": "Conflict",
            }
        }
        self.stub_url('POST', ['tenants'], status_code=409, json=resp_body)

        def create_duplicate_tenant():
            self.client.tenants.create(req_body['tenant']['name'],
                                       req_body['tenant']['description'],
                                       req_body['tenant']['enabled'])

        self.assertRaises(exceptions.Conflict, create_duplicate_tenant)

    def test_delete(self):
        self.stub_url('DELETE', ['tenants', self.ADMIN_ID], status_code=204)
        self.client.tenants.delete(self.ADMIN_ID)

    def test_get(self):
        resp = {'tenant': self.TEST_TENANTS['tenants']['values'][2]}
        self.stub_url('GET', ['tenants', self.ADMIN_ID], json=resp)

        t = self.client.tenants.get(self.ADMIN_ID)
        self.assertIsInstance(t, tenants.Tenant)
        self.assertEqual(t.id, self.ADMIN_ID)
        self.assertEqual(t.name, 'admin')

    def test_list(self):
        self.stub_url('GET', ['tenants'], json=self.TEST_TENANTS)

        tenant_list = self.client.tenants.list()
        [self.assertIsInstance(t, tenants.Tenant) for t in tenant_list]

    def test_list_limit(self):
        self.stub_url('GET', ['tenants'], json=self.TEST_TENANTS)

        tenant_list = self.client.tenants.list(limit=1)
        self.assertQueryStringIs('limit=1')
        [self.assertIsInstance(t, tenants.Tenant) for t in tenant_list]

    def test_list_marker(self):
        self.stub_url('GET', ['tenants'], json=self.TEST_TENANTS)

        tenant_list = self.client.tenants.list(marker=1)
        self.assertQueryStringIs('marker=1')
        [self.assertIsInstance(t, tenants.Tenant) for t in tenant_list]

    def test_list_limit_marker(self):
        self.stub_url('GET', ['tenants'], json=self.TEST_TENANTS)

        tenant_list = self.client.tenants.list(limit=1, marker=1)
        self.assertQueryStringIs('marker=1&limit=1')
        [self.assertIsInstance(t, tenants.Tenant) for t in tenant_list]

    def test_update(self):
        req_body = {
            "tenant": {
                "id": self.EXTRAS_ID,
                "name": "tenantX",
                "description": "I changed you!",
                "enabled": False,
                "extravalue01": "metadataChanged",
                # "extraname": "dontoverwrite!",
            },
        }
        resp_body = {
            "tenant": {
                "name": "tenantX",
                "enabled": False,
                "id": self.EXTRAS_ID,
                "description": "I changed you!",
                "extravalue01": "metadataChanged",
            },
        }

        self.stub_url('POST', ['tenants', self.EXTRAS_ID], json=resp_body)

        tenant = self.client.tenants.update(
            req_body['tenant']['id'],
            req_body['tenant']['name'],
            req_body['tenant']['description'],
            req_body['tenant']['enabled'],
            extravalue01=req_body['tenant']['extravalue01'],
            name="don't overwrite priors")
        self.assertIsInstance(tenant, tenants.Tenant)
        self.assertRequestBodyIs(json=req_body)
        self.assertEqual(tenant.id, self.EXTRAS_ID)
        self.assertEqual(tenant.name, "tenantX")
        self.assertEqual(tenant.description, "I changed you!")
        self.assertFalse(tenant.enabled)
        self.assertEqual(tenant.extravalue01, "metadataChanged")

    def test_update_empty_description(self):
        req_body = {
            "tenant": {
                "id": self.EXTRAS_ID,
                "name": "tenantX",
                "description": "",
                "enabled": False,
            },
        }
        resp_body = {
            "tenant": {
                "name": "tenantX",
                "enabled": False,
                "id": self.EXTRAS_ID,
                "description": "",
            },
        }
        self.stub_url('POST', ['tenants', self.EXTRAS_ID], json=resp_body)

        tenant = self.client.tenants.update(req_body['tenant']['id'],
                                            req_body['tenant']['name'],
                                            req_body['tenant']['description'],
                                            req_body['tenant']['enabled'])
        self.assertIsInstance(tenant, tenants.Tenant)
        self.assertRequestBodyIs(json=req_body)
        self.assertEqual(tenant.id, self.EXTRAS_ID)
        self.assertEqual(tenant.name, "tenantX")
        self.assertEqual(tenant.description, "")
        self.assertFalse(tenant.enabled)

    def test_add_user(self):
        self.stub_url('PUT',
                      ['tenants', self.EXTRAS_ID, 'users', 'foo', 'roles',
                       'OS-KSADM', 'barrr'],
                      status_code=204)

        self.client.tenants.add_user(self.EXTRAS_ID, 'foo', 'barrr')

    def test_remove_user(self):
        self.stub_url('DELETE', ['tenants', self.EXTRAS_ID, 'users',
                                 'foo', 'roles', 'OS-KSADM', 'barrr'],
                      status_code=204)

        self.client.tenants.remove_user(self.EXTRAS_ID, 'foo', 'barrr')

    def test_tenant_add_user(self):
        self.stub_url('PUT', ['tenants', self.EXTRAS_ID, 'users',
                              'foo', 'roles', 'OS-KSADM', 'barrr'],
                      status_code=204)

        req_body = {
            "tenant": {
                "id": self.EXTRAS_ID,
                "name": "tenantX",
                "description": "I changed you!",
                "enabled": False,
            },
        }
        # make tenant object with manager
        tenant = self.client.tenants.resource_class(self.client.tenants,
                                                    req_body['tenant'])
        tenant.add_user('foo', 'barrr')
        self.assertIsInstance(tenant, tenants.Tenant)

    def test_tenant_remove_user(self):
        self.stub_url('DELETE', ['tenants', self.EXTRAS_ID, 'users',
                                 'foo', 'roles', 'OS-KSADM', 'barrr'],
                      status_code=204)

        req_body = {
            "tenant": {
                "id": self.EXTRAS_ID,
                "name": "tenantX",
                "description": "I changed you!",
                "enabled": False,
            },
        }

        # make tenant object with manager
        tenant = self.client.tenants.resource_class(self.client.tenants,
                                                    req_body['tenant'])
        tenant.remove_user('foo', 'barrr')
        self.assertIsInstance(tenant, tenants.Tenant)

    def test_tenant_list_users(self):
        tenant_id = uuid.uuid4().hex
        user_id1 = uuid.uuid4().hex
        user_id2 = uuid.uuid4().hex

        tenant_resp = {
            'tenant': {
                'name': uuid.uuid4().hex,
                'enabled': True,
                'id': tenant_id,
                'description': 'test tenant',
            }
        }

        users_resp = {
            'users': {
                'values': [
                    {
                        'email': uuid.uuid4().hex,
                        'enabled': True,
                        'id': user_id1,
                        'name': uuid.uuid4().hex,
                    },
                    {
                        'email': uuid.uuid4().hex,
                        'enabled': True,
                        'id': user_id2,
                        'name': uuid.uuid4().hex,
                    },
                ]
            }
        }

        self.stub_url('GET', ['tenants', tenant_id], json=tenant_resp)
        self.stub_url('GET',
                      ['tenants', tenant_id, 'users'],
                      json=users_resp)

        tenant = self.client.tenants.get(tenant_id)
        user_objs = tenant.list_users()

        for u in user_objs:
            self.assertIsInstance(u, users.User)

        self.assertEqual(set([user_id1, user_id2]),
                         set([u.id for u in user_objs]))

    def test_list_tenants_use_admin_url(self):
        self.stub_url('GET', ['tenants'], json=self.TEST_TENANTS)

        tenant_list = self.client.tenants.list()
        self.assertEqual(self.TEST_URL + '/tenants',
                         self.requests_mock.last_request.url)
        [self.assertIsInstance(t, tenants.Tenant) for t in tenant_list]

        self.assertEqual(len(self.TEST_TENANTS['tenants']['values']),
                         len(tenant_list))

    def test_list_tenants_fallback_to_auth_url(self):
        new_auth_url = 'http://keystone.test:5000/v2.0'

        token = fixture.V2Token(token_id=self.TEST_TOKEN,
                                user_name=self.TEST_USER,
                                user_id=self.TEST_USER_ID)

        self.stub_auth(base_url=new_auth_url, json=token)
        self.stub_url('GET', ['tenants'], base_url=new_auth_url,
                      json=self.TEST_TENANTS)

        # Creating a HTTPClient not using session is deprecated.
        with self.deprecations.expect_deprecations_here():
            c = client.Client(username=self.TEST_USER,
                              auth_url=new_auth_url,
                              password=uuid.uuid4().hex)

        self.assertIsNone(c.management_url)
        tenant_list = c.tenants.list()
        [self.assertIsInstance(t, tenants.Tenant) for t in tenant_list]

        self.assertEqual(len(self.TEST_TENANTS['tenants']['values']),
                         len(tenant_list))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from keystoneauth1.exceptions import http
from keystoneclient.tests.functional import base
from keystoneclient.tests.functional.v3 import client_fixtures as fixtures


class GroupsTestCase(base.V3ClientTestCase):

    def check_group(self, group, group_ref=None):
        self.assertIsNotNone(group.id)
        self.assertIn('self', group.links)
        self.assertIn('/groups/' + group.id, group.links['self'])

        if group_ref:
            self.assertEqual(group_ref['name'], group.name)
            self.assertEqual(group_ref['domain'], group.domain_id)

            # There is no guarantee description is present in group
            if hasattr(group_ref, 'description'):
                self.assertEqual(group_ref['description'], group.description)

        else:
            # Only check remaining mandatory attributes
            self.assertIsNotNone(group.name)
            self.assertIsNotNone(group.domain_id)

    def test_create_group(self):
        group_ref = {
            'name': fixtures.RESOURCE_NAME_PREFIX + uuid.uuid4().hex,
            'domain': self.project_domain_id,
            'description': uuid.uuid4().hex}

        group = self.client.groups.create(**group_ref)
        self.addCleanup(self.client.groups.delete, group)
        self.check_group(group, group_ref)

    def test_get_group(self):
        group = fixtures.Group(self.client, self.project_domain_id)
        self.useFixture(group)

        group_ret = self.client.groups.get(group.id)
        self.check_group(group_ret, group.ref)

    def test_list_groups(self):
        group_one = fixtures.Group(self.client, self.project_domain_id)
        self.useFixture(group_one)

        group_two = fixtures.Group(self.client, self.project_domain_id)
        self.useFixture(group_two)

        groups = self.client.groups.list()

        # All groups are valid
        for group in groups:
            self.check_group(group)

        self.assertIn(group_one.entity, groups)
        self.assertIn(group_two.entity, groups)

    def test_update_group(self):
        group = fixtures.Group(self.client, self.project_domain_id)
        self.useFixture(group)

        new_name = fixtures.RESOURCE_NAME_PREFIX + uuid.uuid4().hex
        new_description = uuid.uuid4().hex

        group_ret = self.client.groups.update(group.id,
                                              name=new_name,
                                              description=new_description)

        group.ref.update({'name': new_name, 'description': new_description})
        self.check_group(group_ret, group.ref)

    def test_delete_group(self):
        group = self.client.groups.create(name=uuid.uuid4().hex,
                                          domain=self.project_domain_id)

        self.client.groups.delete(group.id)
        self.assertRaises(http.NotFound,
                          self.client.groups.get,
                          group.id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from keystoneclient.tests.unit.v3 import utils
from keystoneclient.v3 import services


class ServiceTests(utils.ClientTestCase, utils.CrudTests):
    def setUp(self):
        super(ServiceTests, self).setUp()
        self.key = 'service'
        self.collection_key = 'services'
        self.model = services.Service
        self.manager = self.client.services

    def new_ref(self, **kwargs):
        kwargs = super(ServiceTests, self).new_ref(**kwargs)
        kwargs.setdefault('name', uuid.uuid4().hex)
        kwargs.setdefault('type', uuid.uuid4().hex)
        kwargs.setdefault('enabled', True)
        return kwargs

    def test_list_filter_name(self):
        filter_name = uuid.uuid4().hex
        expected_query = {'name': filter_name}
        super(ServiceTests, self).test_list(expected_query=expected_query,
                                            name=filter_name)

    def test_list_filter_type(self):
        filter_type = uuid.uuid4().hex
        expected_query = {'type': filter_type}
        super(ServiceTests, self).test_list(expected_query=expected_query,
                                            type=filter_type)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from keystoneauth1 import exceptions
import testresources

from keystoneclient import access
from keystoneclient.tests.unit import client_fixtures
from keystoneclient.tests.unit.v3 import utils


class TokenTests(utils.ClientTestCase, testresources.ResourcedTestCase):

    resources = [('examples', client_fixtures.EXAMPLES_RESOURCE)]

    def test_revoke_token_with_token_id(self):
        token_id = uuid.uuid4().hex
        self.stub_url('DELETE', ['/auth/tokens'], status_code=204)
        self.client.tokens.revoke_token(token_id)
        self.assertRequestHeaderEqual('X-Subject-Token', token_id)

    def test_revoke_token_with_access_info_instance(self):
        token_id = uuid.uuid4().hex
        token_ref = self.examples.TOKEN_RESPONSES[
            self.examples.v3_UUID_TOKEN_DEFAULT]
        token = access.AccessInfoV3(token_id, token_ref['token'])
        self.stub_url('DELETE', ['/auth/tokens'], status_code=204)
        self.client.tokens.revoke_token(token)
        self.assertRequestHeaderEqual('X-Subject-Token', token_id)

    def test_get_revoked(self):
        sample_revoked_response = {'signed': '-----BEGIN CMS-----\nMIIB...'}
        self.stub_url('GET', ['auth', 'tokens', 'OS-PKI', 'revoked'],
                      json=sample_revoked_response)
        resp = self.client.tokens.get_revoked()
        self.assertQueryStringIs()
        self.assertEqual(sample_revoked_response, resp)

    def test_get_revoked_audit_id_only(self):
        # When get_revoked(audit_id_only=True) then ?audit_id_only is set on
        # the request.
        sample_revoked_response = {
            'revoked': [
                {
                    'audit_id': uuid.uuid4().hex,
                    'expires': '2016-01-21T15:53:52Z',
                },
            ],
        }
        self.stub_url('GET', ['auth', 'tokens', 'OS-PKI', 'revoked'],
                      json=sample_revoked_response)
        resp = self.client.tokens.get_revoked(audit_id_only=True)
        self.assertQueryStringIs('audit_id_only')
        self.assertEqual(sample_revoked_response, resp)

    def test_validate_token_with_token_id(self):
        # Can validate a token passing a string token ID.
        token_id = uuid.uuid4().hex
        token_ref = self.examples.TOKEN_RESPONSES[
            self.examples.v3_UUID_TOKEN_DEFAULT]
        self.stub_url('GET', ['auth', 'tokens'],
                      headers={'X-Subject-Token': token_id, }, json=token_ref)

        token_data = self.client.tokens.get_token_data(token_id)
        self.assertEqual(token_data, token_ref)

        access_info = self.client.tokens.validate(token_id)

        self.assertRequestHeaderEqual('X-Subject-Token', token_id)
        self.assertIsInstance(access_info, access.AccessInfoV3)
        self.assertEqual(token_id, access_info.auth_token)

    def test_validate_token_with_access_info(self):
        # Can validate a token passing an access info.
        token_id = uuid.uuid4().hex
        token_ref = self.examples.TOKEN_RESPONSES[
            self.examples.v3_UUID_TOKEN_DEFAULT]
        token = access.AccessInfoV3(token_id, token_ref['token'])
        self.stub_url('GET', ['auth', 'tokens'],
                      headers={'X-Subject-Token': token_id, }, json=token_ref)
        access_info = self.client.tokens.validate(token)

        self.assertRequestHeaderEqual('X-Subject-Token', token_id)
        self.assertIsInstance(access_info, access.AccessInfoV3)
        self.assertEqual(token_id, access_info.auth_token)

    def test_validate_token_invalid(self):
        # When the token is invalid the server typically returns a 404.
        token_id = uuid.uuid4().hex
        self.stub_url('GET', ['auth', 'tokens'], status_code=404)

        self.assertRaises(exceptions.NotFound,
                          self.client.tokens.get_token_data, token_id)
        self.assertRaises(exceptions.NotFound,
                          self.client.tokens.validate, token_id)

    def test_validate_token_catalog(self):
        # Can validate a token and a catalog is requested by default.
        token_id = uuid.uuid4().hex
        token_ref = self.examples.TOKEN_RESPONSES[
            self.examples.v3_UUID_TOKEN_DEFAULT]
        self.stub_url('GET', ['auth', 'tokens'],
                      headers={'X-Subject-Token': token_id, }, json=token_ref)

        token_data = self.client.tokens.get_token_data(token_id)
        self.assertQueryStringIs()
        self.assertIn('catalog', token_data['token'])

        access_info = self.client.tokens.validate(token_id)

        self.assertQueryStringIs()
        self.assertTrue(access_info.has_service_catalog())

    def test_validate_token_nocatalog(self):
        # Can validate a token and request no catalog.
        token_id = uuid.uuid4().hex
        token_ref = self.examples.TOKEN_RESPONSES[
            self.examples.v3_UUID_TOKEN_UNSCOPED]
        self.stub_url('GET', ['auth', 'tokens'],
                      headers={'X-Subject-Token': token_id, }, json=token_ref)

        token_data = self.client.tokens.get_token_data(token_id)
        self.assertQueryStringIs()
        self.assertNotIn('catalog', token_data['token'])

        access_info = self.client.tokens.validate(token_id,
                                                  include_catalog=False)

        self.assertQueryStringIs('nocatalog')
        self.assertFalse(access_info.has_service_catalog())

    def test_validate_token_allow_expired(self):
        token_id = uuid.uuid4().hex
        token_ref = self.examples.TOKEN_RESPONSES[
            self.examples.v3_UUID_TOKEN_UNSCOPED]
        self.stub_url('GET', ['auth', 'tokens'],
                      headers={'X-Subject-Token': token_id, }, json=token_ref)

        self.client.tokens.validate(token_id)
        self.assertQueryStringIs()

        self.client.tokens.validate(token_id, allow_expired=True)
        self.assertQueryStringIs('allow_expired=1')


def load_tests(loader, tests, pattern):
    return testresources.OptimisingTestSuite(tests)



# Copyright 2014 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils

from keystoneclient.generic import client
from keystoneclient.tests.unit import utils

BASE_HOST = 'http://keystone.example.com'
BASE_URL = "%s:5000/" % BASE_HOST
V2_URL = "%sv2.0" % BASE_URL

EXTENSION_NAMESPACE = ("https://docs.openstack.org/identity/api/ext/OS-FAKE/"
                       "v1.0")
EXTENSION_DESCRIBED = {"href": "https://github.com/openstack/identity-api",
                       "rel": "describedby",
                       "type": "text/html"}

EXTENSION_ALIAS_FOO = "OS-FAKE-FOO"
EXTENSION_NAME_FOO = "OpenStack Keystone Fake Extension Foo"
EXTENSION_FOO = {"alias": EXTENSION_ALIAS_FOO,
                 "description": "Fake Foo extension to V2.0 API.",
                 "links": [EXTENSION_DESCRIBED],
                 "name": EXTENSION_NAME_FOO,
                 "namespace": EXTENSION_NAMESPACE,
                 "updated": '2014-01-08T00:00:00Z'}

EXTENSION_ALIAS_BAR = "OS-FAKE-BAR"
EXTENSION_NAME_BAR = "OpenStack Keystone Fake Extension Bar"
EXTENSION_BAR = {"alias": EXTENSION_ALIAS_BAR,
                 "description": "Fake Bar extension to V2.0 API.",
                 "links": [EXTENSION_DESCRIBED],
                 "name": EXTENSION_NAME_BAR,
                 "namespace": EXTENSION_NAMESPACE,
                 "updated": '2014-01-08T00:00:00Z'}


def _create_extension_list(extensions):
    return jsonutils.dumps({'extensions': {'values': extensions}})


EXTENSION_LIST = _create_extension_list([EXTENSION_FOO, EXTENSION_BAR])


class ClientDiscoveryTests(utils.TestCase):

    def test_discover_extensions_v2(self):
        self.requests_mock.get("%s/extensions" % V2_URL, text=EXTENSION_LIST)
        # Creating a HTTPClient not using session is deprecated.
        # creating a generic client at all is deprecated.
        with self.deprecations.expect_deprecations_here():
            extensions = client.Client().discover_extensions(url=V2_URL)
        self.assertIn(EXTENSION_ALIAS_FOO, extensions)
        self.assertEqual(extensions[EXTENSION_ALIAS_FOO], EXTENSION_NAME_FOO)
        self.assertIn(EXTENSION_ALIAS_BAR, extensions)
        self.assertEqual(extensions[EXTENSION_ALIAS_BAR], EXTENSION_NAME_BAR)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import uuid

from keystoneauth1 import fixture


def unscoped_token():
    return fixture.V2Token(token_id='3e2813b7ba0b4006840c3825860b86ed',
                           expires='2012-10-03T16:58:01Z',
                           user_id='c4da488862bd435c9e6c0275a0d0e49a',
                           user_name='exampleuser')


def project_scoped_token():
    _TENANT_ID = '225da22d3ce34b15877ea70b2a575f58'

    f = fixture.V2Token(token_id='04c7d5ffaeef485f9dc69c06db285bdb',
                        expires='2012-10-03T16:53:36Z',
                        tenant_id='225da22d3ce34b15877ea70b2a575f58',
                        tenant_name='exampleproject',
                        user_id='c4da488862bd435c9e6c0275a0d0e49a',
                        user_name='exampleuser',
                        audit_chain_id=uuid.uuid4().hex)

    f.add_role(id='member_id', name='Member')

    s = f.add_service('volume', 'Volume Service')
    s.add_endpoint(public='http://public.com:8776/v1/%s' % _TENANT_ID,
                   admin='http://admin:8776/v1/%s' % _TENANT_ID,
                   internal='http://internal:8776/v1/%s' % _TENANT_ID,
                   region='RegionOne')

    s = f.add_service('image', 'Image Service')
    s.add_endpoint(public='http://public.com:9292/v1',
                   admin='http://admin:9292/v1',
                   internal='http://internal:9292/v1',
                   region='RegionOne')

    s = f.add_service('compute', 'Compute Service')
    s.add_endpoint(public='http://public.com:8774/v2/%s' % _TENANT_ID,
                   admin='http://admin:8774/v2/%s' % _TENANT_ID,
                   internal='http://internal:8774/v2/%s' % _TENANT_ID,
                   region='RegionOne')

    s = f.add_service('ec2', 'EC2 Service')
    s.add_endpoint(public='http://public.com:8773/services/Cloud',
                   admin='http://admin:8773/services/Admin',
                   internal='http://internal:8773/services/Cloud',
                   region='RegionOne')

    s = f.add_service('identity', 'Identity Service')
    s.add_endpoint(public='http://public.com:5000/v2.0',
                   admin='http://admin:35357/v2.0',
                   internal='http://internal:5000/v2.0',
                   region='RegionOne')

    return f


def auth_response_body():
    f = fixture.V2Token(token_id='ab48a9efdfedb23ty3494',
                        expires='2010-11-01T03:32:15-05:00',
                        tenant_id='345',
                        tenant_name='My Project',
                        user_id='123',
                        user_name='jqsmith',
                        audit_chain_id=uuid.uuid4().hex)

    f.add_role(id='234', name='compute:admin')
    role = f.add_role(id='235', name='object-store:admin')
    role['tenantId'] = '1'

    s = f.add_service('compute', 'Cloud Servers')
    endpoint = s.add_endpoint(public='https://compute.north.host/v1/1234',
                              internal='https://compute.north.host/v1/1234',
                              region='North')
    endpoint['tenantId'] = '1'
    endpoint['versionId'] = '1.0'
    endpoint['versionInfo'] = 'https://compute.north.host/v1.0/'
    endpoint['versionList'] = 'https://compute.north.host/'

    endpoint = s.add_endpoint(public='https://compute.north.host/v1.1/3456',
                              internal='https://compute.north.host/v1.1/3456',
                              region='North')
    endpoint['tenantId'] = '2'
    endpoint['versionId'] = '1.1'
    endpoint['versionInfo'] = 'https://compute.north.host/v1.1/'
    endpoint['versionList'] = 'https://compute.north.host/'

    s = f.add_service('object-store', 'Cloud Files')
    endpoint = s.add_endpoint(public='https://swift.north.host/v1/blah',
                              internal='https://swift.north.host/v1/blah',
                              region='South')
    endpoint['tenantId'] = '11'
    endpoint['versionId'] = '1.0'
    endpoint['versionInfo'] = 'uri'
    endpoint['versionList'] = 'uri'

    endpoint = s.add_endpoint(public='https://swift.north.host/v1.1/blah',
                              internal='https://compute.north.host/v1.1/blah',
                              region='South')
    endpoint['tenantId'] = '2'
    endpoint['versionId'] = '1.1'
    endpoint['versionInfo'] = 'https://swift.north.host/v1.1/'
    endpoint['versionList'] = 'https://swift.north.host/'

    s = f.add_service('image', 'Image Servers')
    s.add_endpoint(public='https://image.north.host/v1/',
                   internal='https://image-internal.north.host/v1/',
                   region='North')
    s.add_endpoint(public='https://image.south.host/v1/',
                   internal='https://image-internal.south.host/v1/',
                   region='South')

    return f



# Copyright 2014 IBM Corp.
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testresources

from keystoneclient.tests.unit import client_fixtures
from keystoneclient.tests.unit.v2_0 import utils


class CertificateTests(utils.ClientTestCase, testresources.ResourcedTestCase):

    resources = [('examples', client_fixtures.EXAMPLES_RESOURCE)]

    def test_get_ca_certificate(self):
        self.stub_url('GET', ['certificates', 'ca'],
                      headers={'Content-Type': 'text/html; charset=UTF-8'},
                      text=self.examples.SIGNING_CA)
        res = self.client.certificates.get_ca_certificate()
        self.assertEqual(self.examples.SIGNING_CA, res)

    def test_get_signing_certificate(self):
        self.stub_url('GET', ['certificates', 'signing'],
                      headers={'Content-Type': 'text/html; charset=UTF-8'},
                      text=self.examples.SIGNING_CERT)
        res = self.client.certificates.get_signing_certificate()
        self.assertEqual(self.examples.SIGNING_CERT, res)


def load_tests(loader, tests, pattern):
    return testresources.OptimisingTestSuite(tests)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import argparse
import os

from debtcollector import removals

from keystoneclient.auth import base


@removals.remove(
    message='keystoneclient auth plugins are deprecated. Use keystoneauth.',
    version='2.1.0',
    removal_version='3.0.0'
)
def register_argparse_arguments(parser, argv, default=None):
    """Register CLI options needed to create a plugin.

    The function inspects the provided arguments so that it can also register
    the options required for that specific plugin if available.

    :param argparse.ArgumentParser: the parser to attach argparse options to.
    :param List argv: the arguments provided to the application.
    :param str/class default: a default plugin name or a plugin object to use
                              if one isn't specified by the CLI. default: None.

    :returns: The plugin class that will be loaded or None if not provided.
    :rtype: :py:class:`keystoneclient.auth.BaseAuthPlugin`

    :raises keystoneclient.exceptions.NoMatchingPlugin: if a plugin cannot be
                                                        created.
    """
    in_parser = argparse.ArgumentParser(add_help=False)
    env_plugin = os.environ.get('OS_AUTH_PLUGIN', default)
    for p in (in_parser, parser):
        p.add_argument('--os-auth-plugin',
                       metavar='<name>',
                       default=env_plugin,
                       help='The auth plugin to load')

    options, _args = in_parser.parse_known_args(argv)

    if not options.os_auth_plugin:
        return None

    if isinstance(options.os_auth_plugin, type):
        msg = 'Default Authentication options'
        plugin = options.os_auth_plugin
    else:
        msg = 'Options specific to the %s plugin.' % options.os_auth_plugin
        plugin = base.get_plugin_class(options.os_auth_plugin)

    group = parser.add_argument_group('Authentication Options', msg)
    plugin.register_argparse_arguments(group)
    return plugin


@removals.remove(
    message='keystoneclient auth plugins are deprecated. Use keystoneauth.',
    version='2.1.0',
    removal_version='3.0.0'
)
def load_from_argparse_arguments(namespace, **kwargs):
    """Retrieve the created plugin from the completed argparse results.

    Loads and creates the auth plugin from the information parsed from the
    command line by argparse.

    :param Namespace namespace: The result from CLI parsing.

    :returns: An auth plugin, or None if a name is not provided.
    :rtype: :py:class:`keystoneclient.auth.BaseAuthPlugin`

    :raises keystoneclient.exceptions.NoMatchingPlugin: if a plugin cannot be
                                                        created.
    """
    if not namespace.os_auth_plugin:
        return None

    if isinstance(namespace.os_auth_plugin, type):
        plugin = namespace.os_auth_plugin
    else:
        plugin = base.get_plugin_class(namespace.os_auth_plugin)

    return plugin.load_from_argparse_arguments(namespace, **kwargs)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from keystoneclient.v3.contrib.federation import domains
from keystoneclient.v3.contrib.federation import identity_providers
from keystoneclient.v3.contrib.federation import mappings
from keystoneclient.v3.contrib.federation import projects
from keystoneclient.v3.contrib.federation import protocols
from keystoneclient.v3.contrib.federation import saml
from keystoneclient.v3.contrib.federation import service_providers


class FederationManager(object):
    def __init__(self, api):
        self.identity_providers = identity_providers.IdentityProviderManager(
            api)
        self.mappings = mappings.MappingManager(api)
        self.protocols = protocols.ProtocolManager(api)
        self.projects = projects.ProjectManager(api)
        self.domains = domains.DomainManager(api)
        self.saml = saml.SamlManager(api)
        self.service_providers = service_providers.ServiceProviderManager(api)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from keystoneclient import base
from keystoneclient import exceptions
from keystoneclient.i18n import _


class DomainConfig(base.Resource):
    """An object representing a domain config association.

    This resource object does not necessarily contain fixed attributes, as new
    attributes are added in the server, they are supported here directly.
    The currently supported configs are `identity` and `ldap`.

    """

    pass


class DomainConfigManager(base.Manager):
    """Manager class for manipulating domain config associations."""

    resource_class = DomainConfig
    key = 'config'

    def build_url(self, domain):
        return '/domains/%s/config' % base.getid(domain)

    def create(self, domain, config):
        """Create a config for a domain.

        :param domain: the domain where the config is going to be applied.
        :type domain: str or :py:class:`keystoneclient.v3.domains.Domain`

        :param dict config: a dictionary of domain configurations.

        Example of the ``config`` parameter::

            {
                 "identity": {
                     "driver": "ldap"
                 },
                 "ldap": {
                     "url": "ldap://myldap.com:389/",
                     "user_tree_dn": "ou=Users,dc=my_new_root,dc=org"
                 }
            }

        :returns: the created domain config returned from server.
        :rtype: :class:`keystoneclient.v3.domain_configs.DomainConfig`

        """
        base_url = self.build_url(domain)
        body = {self.key: config}
        return super(DomainConfigManager, self)._put(
            base_url, body=body, response_key=self.key)

    def get(self, domain):
        """Get a config for a domain.

        :param domain: the domain for which the config is defined.
        :type domain: str or :py:class:`keystoneclient.v3.domains.Domain`

        :returns: the domain config returned from server.
        :rtype: :class:`keystoneclient.v3.domain_configs.DomainConfig`

        """
        base_url = self.build_url(domain)
        return super(DomainConfigManager, self)._get(base_url, self.key)

    def update(self, domain, config):
        """Update a config for a domain.

        :param domain: the domain where the config is going to be updated.
        :type domain: str or :py:class:`keystoneclient.v3.domains.Domain`

        :param dict config: a dictionary of domain configurations.

        Example of the ``config`` parameter::

            {
                 "identity": {
                     "driver": "ldap"
                 },
                 "ldap": {
                     "url": "ldap://myldap.com:389/",
                     "user_tree_dn": "ou=Users,dc=my_new_root,dc=org"
                 }
            }

        :returns: the updated domain config returned from server.
        :rtype: :class:`keystoneclient.v3.domain_configs.DomainConfig`

        """
        base_url = self.build_url(domain)
        body = {self.key: config}
        return super(DomainConfigManager, self)._patch(
            base_url, body=body, response_key=self.key)

    def delete(self, domain):
        """Delete a config for a domain.

        :param domain: the domain which the config will be deleted on
                       the server.
        :type domain: str or :class:`keystoneclient.v3.domains.Domain`

        :returns: Response object with 204 status.
        :rtype: :class:`requests.models.Response`

        """
        base_url = self.build_url(domain)
        return super(DomainConfigManager, self)._delete(url=base_url)

    def find(self, **kwargs):
        raise exceptions.MethodNotImplemented(
            _('Find not supported for domain configs'))

    def list(self, **kwargs):
        raise exceptions.MethodNotImplemented(
            _('List not supported for domain configs'))



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from keystoneclient.i18n import _
from keystoneclient.v3.contrib.oauth1 import access_tokens
from keystoneclient.v3.contrib.oauth1 import consumers
from keystoneclient.v3.contrib.oauth1 import request_tokens


def create_oauth_manager(self):
    # NOTE(stevemar): Attempt to import the oauthlib package at this point.
    try:
        import oauthlib  # noqa
    # NOTE(stevemar): Return an object instead of raising an exception here,
    # this will allow users to see an exception only when trying to access the
    # oauth portions of client. Otherwise an exception would be raised
    # when the client is created.
    except ImportError:
        return OAuthManagerOptionalImportProxy()
    else:
        return OAuthManager(self)


class OAuthManager(object):
    def __init__(self, api):
        self.access_tokens = access_tokens.AccessTokenManager(api)
        self.consumers = consumers.ConsumerManager(api)
        self.request_tokens = request_tokens.RequestTokenManager(api)


class OAuthManagerOptionalImportProxy(object):
    """Act as a proxy manager in case oauthlib is no installed.

    This class will only be created if oauthlib is not in the system,
    trying to access any of the attributes in name (access_tokens,
    consumers, request_tokens), will result in a NotImplementedError,
    and a message.

    >>> manager.access_tokens.blah
    NotImplementedError: To use 'access_tokens' oauthlib must be installed

    Otherwise, if trying to access an attribute other than the ones in name,
    the manager will state that the attribute does not exist.

    >>> manager.dne.blah
    AttributeError: 'OAuthManagerOptionalImportProxy' object has no
    attribute 'dne'
    """

    def __getattribute__(self, name):
        """Return error when name is related to oauthlib and not exist."""
        if name in ('access_tokens', 'consumers', 'request_tokens'):
            raise NotImplementedError(
                _('To use %r oauthlib must be installed') % name)
        return super(OAuthManagerOptionalImportProxy,
                     self).__getattribute__(name)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import uuid

from keystoneclient.tests.unit.v2_0 import utils
from keystoneclient.v2_0 import services


class ServiceTests(utils.ClientTestCase):
    def setUp(self):
        super(ServiceTests, self).setUp()

        self.NOVA_SERVICE_ID = uuid.uuid4().hex
        self.KEYSTONE_SERVICE_ID = uuid.uuid4().hex

        self.TEST_SERVICES = {
            "OS-KSADM:services": {
                "values": [
                    {
                        "name": "nova",
                        "type": "compute",
                        "description": "Nova-compatible service.",
                        "id": self.NOVA_SERVICE_ID
                    },
                    {
                        "name": "keystone",
                        "type": "identity",
                        "description": "Keystone-compatible service.",
                        "id": self.KEYSTONE_SERVICE_ID
                    },
                ],
            },
        }

    def test_create_with_description(self):
        req_body = {
            "OS-KSADM:service": {
                "name": "swift",
                "type": "object-store",
                "description": "Swift-compatible service.",
            }
        }
        service_id = uuid.uuid4().hex
        resp_body = {
            "OS-KSADM:service": {
                "name": "swift",
                "type": "object-store",
                "description": "Swift-compatible service.",
                "id": service_id,
            }
        }
        self.stub_url('POST', ['OS-KSADM', 'services'], json=resp_body)

        service = self.client.services.create(
            req_body['OS-KSADM:service']['name'],
            req_body['OS-KSADM:service']['type'],
            req_body['OS-KSADM:service']['description'])
        self.assertIsInstance(service, services.Service)
        self.assertEqual(service.id, service_id)
        self.assertEqual(service.name, req_body['OS-KSADM:service']['name'])
        self.assertEqual(service.description,
                         req_body['OS-KSADM:service']['description'])
        self.assertRequestBodyIs(json=req_body)

    def test_create_without_description(self):
        req_body = {
            "OS-KSADM:service": {
                "name": "swift",
                "type": "object-store",
                "description": None,
            }
        }
        service_id = uuid.uuid4().hex
        resp_body = {
            "OS-KSADM:service": {
                "name": "swift",
                "type": "object-store",
                "id": service_id,
                "description": None,
            }
        }
        self.stub_url('POST', ['OS-KSADM', 'services'], json=resp_body)

        service = self.client.services.create(
            req_body['OS-KSADM:service']['name'],
            req_body['OS-KSADM:service']['type'],
            req_body['OS-KSADM:service']['description'])
        self.assertIsInstance(service, services.Service)
        self.assertEqual(service.id, service_id)
        self.assertEqual(service.name, req_body['OS-KSADM:service']['name'])
        self.assertIsNone(service.description)
        self.assertRequestBodyIs(json=req_body)

    def test_delete(self):
        self.stub_url('DELETE',
                      ['OS-KSADM', 'services', self.NOVA_SERVICE_ID],
                      status_code=204)

        self.client.services.delete(self.NOVA_SERVICE_ID)

    def test_get(self):
        test_services = self.TEST_SERVICES['OS-KSADM:services']['values'][0]

        self.stub_url('GET', ['OS-KSADM', 'services', self.NOVA_SERVICE_ID],
                      json={'OS-KSADM:service': test_services})

        service = self.client.services.get(self.NOVA_SERVICE_ID)
        self.assertIsInstance(service, services.Service)
        self.assertEqual(service.id, self.NOVA_SERVICE_ID)
        self.assertEqual(service.name, 'nova')
        self.assertEqual(service.type, 'compute')

    def test_list(self):
        self.stub_url('GET', ['OS-KSADM', 'services'],
                      json=self.TEST_SERVICES)

        service_list = self.client.services.list()
        [self.assertIsInstance(r, services.Service)
         for r in service_list]



# Copyright 2013 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import contextlib
import os
import uuid
import warnings

import fixtures
from keystoneauth1 import fixture
from keystoneauth1 import identity as ksa_identity
from keystoneauth1 import session as ksa_session
from oslo_serialization import jsonutils
from oslo_utils import timeutils
import testresources

from keystoneclient.auth import identity as ksc_identity
from keystoneclient.common import cms
from keystoneclient import session as ksc_session
from keystoneclient import utils
from keystoneclient.v2_0 import client as v2_client
from keystoneclient.v3 import client as v3_client


TEST_ROOT_URL = 'http://127.0.0.1:5000/'

TESTDIR = os.path.dirname(os.path.abspath(__file__))
ROOTDIR = os.path.normpath(os.path.join(TESTDIR, '..', '..', '..'))
CERTDIR = os.path.join(ROOTDIR, 'examples', 'pki', 'certs')
CMSDIR = os.path.join(ROOTDIR, 'examples', 'pki', 'cms')
KEYDIR = os.path.join(ROOTDIR, 'examples', 'pki', 'private')


class BaseFixture(fixtures.Fixture):

    TEST_ROOT_URL = TEST_ROOT_URL

    def __init__(self, requests, deprecations):
        super(BaseFixture, self).__init__()
        self.requests = requests
        self.deprecations = deprecations
        self.user_id = uuid.uuid4().hex
        self.client = self.new_client()


class BaseV2(BaseFixture):

    TEST_URL = '%s%s' % (TEST_ROOT_URL, 'v2.0')


class OriginalV2(BaseV2):

    def new_client(self):
        # Creating a Client not using session is deprecated.
        with self.deprecations.expect_deprecations_here():
            return v2_client.Client(username=uuid.uuid4().hex,
                                    user_id=self.user_id,
                                    token=uuid.uuid4().hex,
                                    tenant_name=uuid.uuid4().hex,
                                    auth_url=self.TEST_URL,
                                    endpoint=self.TEST_URL)


class KscSessionV2(BaseV2):

    def new_client(self):
        t = fixture.V2Token(user_id=self.user_id)
        t.set_scope()

        s = t.add_service('identity')
        s.add_endpoint(self.TEST_URL)

        d = fixture.V2Discovery(self.TEST_URL)

        self.requests.register_uri('POST', self.TEST_URL + '/tokens', json=t)

        # NOTE(jamielennox): Because of the versioned URL hack here even though
        # the V2 URL will be in the service catalog it will be the root URL
        # that will be queried for discovery.
        self.requests.register_uri('GET', self.TEST_ROOT_URL,
                                   json={'version': d})

        with self.deprecations.expect_deprecations_here():
            a = ksc_identity.V2Password(username=uuid.uuid4().hex,
                                        password=uuid.uuid4().hex,
                                        auth_url=self.TEST_URL)

            s = ksc_session.Session(auth=a)

        return v2_client.Client(session=s)


class KsaSessionV2(BaseV2):

    def new_client(self):
        t = fixture.V2Token(user_id=self.user_id)
        t.set_scope()

        s = t.add_service('identity')
        s.add_endpoint(self.TEST_URL)

        d = fixture.V2Discovery(self.TEST_URL)

        self.requests.register_uri('POST', self.TEST_URL + '/tokens', json=t)

        # NOTE(jamielennox): Because of the versioned URL hack here even though
        # the V2 URL will be in the service catalog it will be the root URL
        # that will be queried for discovery.
        self.requests.register_uri('GET', self.TEST_ROOT_URL,
                                   json={'version': d})

        a = ksa_identity.V2Password(username=uuid.uuid4().hex,
                                    password=uuid.uuid4().hex,
                                    auth_url=self.TEST_URL)
        s = ksa_session.Session(auth=a)
        return v2_client.Client(session=s)


class BaseV3(BaseFixture):

    TEST_URL = '%s%s' % (TEST_ROOT_URL, 'v3')


class OriginalV3(BaseV3):

    def new_client(self):
        # Creating a Client not using session is deprecated.
        with self.deprecations.expect_deprecations_here():
            return v3_client.Client(username=uuid.uuid4().hex,
                                    user_id=self.user_id,
                                    token=uuid.uuid4().hex,
                                    tenant_name=uuid.uuid4().hex,
                                    auth_url=self.TEST_URL,
                                    endpoint=self.TEST_URL)


class KscSessionV3(BaseV3):

    def new_client(self):
        t = fixture.V3Token(user_id=self.user_id)
        t.set_project_scope()

        s = t.add_service('identity')
        s.add_standard_endpoints(public=self.TEST_URL,
                                 admin=self.TEST_URL)

        d = fixture.V3Discovery(self.TEST_URL)

        headers = {'X-Subject-Token': uuid.uuid4().hex}
        self.requests.register_uri('POST',
                                   self.TEST_URL + '/auth/tokens',
                                   headers=headers,
                                   json=t)
        self.requests.register_uri('GET', self.TEST_URL, json={'version': d})

        with self.deprecations.expect_deprecations_here():
            a = ksc_identity.V3Password(username=uuid.uuid4().hex,
                                        password=uuid.uuid4().hex,
                                        user_domain_id=uuid.uuid4().hex,
                                        auth_url=self.TEST_URL)

            s = ksc_session.Session(auth=a)

        return v3_client.Client(session=s)


class KsaSessionV3(BaseV3):

    def new_client(self):
        t = fixture.V3Token(user_id=self.user_id)
        t.set_project_scope()

        s = t.add_service('identity')
        s.add_standard_endpoints(public=self.TEST_URL,
                                 admin=self.TEST_URL)

        d = fixture.V3Discovery(self.TEST_URL)

        headers = {'X-Subject-Token': uuid.uuid4().hex}
        self.requests.register_uri('POST',
                                   self.TEST_URL + '/auth/tokens',
                                   headers=headers,
                                   json=t)
        self.requests.register_uri('GET', self.TEST_URL, json={'version': d})

        a = ksa_identity.V3Password(username=uuid.uuid4().hex,
                                    password=uuid.uuid4().hex,
                                    user_domain_id=uuid.uuid4().hex,
                                    auth_url=self.TEST_URL)
        s = ksa_session.Session(auth=a)
        return v3_client.Client(session=s)


def _hash_signed_token_safe(signed_text, **kwargs):
    if isinstance(signed_text, str):
        signed_text = signed_text.encode('utf-8')
    return utils.hash_signed_token(signed_text, **kwargs)


class Examples(fixtures.Fixture):
    """Example tokens and certs loaded from the examples directory.

    To use this class correctly, the module needs to override the test suite
    class to use testresources.OptimisingTestSuite (otherwise the files will
    be read on every test). This is done by defining a load_tests function
    in the module, like this:

    def load_tests(loader, tests, pattern):
        return testresources.OptimisingTestSuite(tests)

    (see http://docs.python.org/2/library/unittest.html#load-tests-protocol )

    """

    def setUp(self):
        super(Examples, self).setUp()

        # The data for several tests are signed using openssl and are stored in
        # files in the signing subdirectory.  In order to keep the values
        # consistent between the tests and the signed documents, we read them
        # in for use in the tests.
        with open(os.path.join(CMSDIR, 'auth_token_scoped.json')) as f:
            self.TOKEN_SCOPED_DATA = cms.cms_to_token(f.read())

        with open(os.path.join(CMSDIR, 'auth_token_scoped.pem')) as f:
            self.SIGNED_TOKEN_SCOPED = cms.cms_to_token(f.read())
        self.SIGNED_TOKEN_SCOPED_HASH = _hash_signed_token_safe(
            self.SIGNED_TOKEN_SCOPED)
        self.SIGNED_TOKEN_SCOPED_HASH_SHA256 = _hash_signed_token_safe(
            self.SIGNED_TOKEN_SCOPED, mode='sha256')
        with open(os.path.join(CMSDIR, 'auth_token_unscoped.pem')) as f:
            self.SIGNED_TOKEN_UNSCOPED = cms.cms_to_token(f.read())
        with open(os.path.join(CMSDIR, 'auth_v3_token_scoped.pem')) as f:
            self.SIGNED_v3_TOKEN_SCOPED = cms.cms_to_token(f.read())
        self.SIGNED_v3_TOKEN_SCOPED_HASH = _hash_signed_token_safe(
            self.SIGNED_v3_TOKEN_SCOPED)
        self.SIGNED_v3_TOKEN_SCOPED_HASH_SHA256 = _hash_signed_token_safe(
            self.SIGNED_v3_TOKEN_SCOPED, mode='sha256')
        with open(os.path.join(CMSDIR, 'auth_token_revoked.pem')) as f:
            self.REVOKED_TOKEN = cms.cms_to_token(f.read())
        with open(os.path.join(CMSDIR, 'auth_token_scoped_expired.pem')) as f:
            self.SIGNED_TOKEN_SCOPED_EXPIRED = cms.cms_to_token(f.read())
        with open(os.path.join(CMSDIR, 'auth_v3_token_revoked.pem')) as f:
            self.REVOKED_v3_TOKEN = cms.cms_to_token(f.read())
        with open(os.path.join(CMSDIR, 'auth_token_scoped.pkiz')) as f:
            self.SIGNED_TOKEN_SCOPED_PKIZ = cms.cms_to_token(f.read())
        with open(os.path.join(CMSDIR, 'auth_token_unscoped.pkiz')) as f:
            self.SIGNED_TOKEN_UNSCOPED_PKIZ = cms.cms_to_token(f.read())
        with open(os.path.join(CMSDIR, 'auth_v3_token_scoped.pkiz')) as f:
            self.SIGNED_v3_TOKEN_SCOPED_PKIZ = cms.cms_to_token(f.read())
        with open(os.path.join(CMSDIR, 'auth_token_revoked.pkiz')) as f:
            self.REVOKED_TOKEN_PKIZ = cms.cms_to_token(f.read())
        with open(os.path.join(CMSDIR,
                               'auth_token_scoped_expired.pkiz')) as f:
            self.SIGNED_TOKEN_SCOPED_EXPIRED_PKIZ = cms.cms_to_token(f.read())
        with open(os.path.join(CMSDIR, 'auth_v3_token_revoked.pkiz')) as f:
            self.REVOKED_v3_TOKEN_PKIZ = cms.cms_to_token(f.read())
        with open(os.path.join(CMSDIR, 'revocation_list.json')) as f:
            self.REVOCATION_LIST = jsonutils.loads(f.read())
        with open(os.path.join(CMSDIR, 'revocation_list.pem')) as f:
            self.SIGNED_REVOCATION_LIST = jsonutils.dumps({'signed': f.read()})

        self.SIGNING_CERT_FILE = os.path.join(CERTDIR, 'signing_cert.pem')
        with open(self.SIGNING_CERT_FILE) as f:
            self.SIGNING_CERT = f.read()

        self.KERBEROS_BIND = 'USER@REALM'

        self.SIGNING_KEY_FILE = os.path.join(KEYDIR, 'signing_key.pem')
        with open(self.SIGNING_KEY_FILE) as f:
            self.SIGNING_KEY = f.read()

        self.SIGNING_CA_FILE = os.path.join(CERTDIR, 'cacert.pem')
        with open(self.SIGNING_CA_FILE) as f:
            self.SIGNING_CA = f.read()

        self.UUID_TOKEN_DEFAULT = "ec6c0710ec2f471498484c1b53ab4f9d"
        self.UUID_TOKEN_NO_SERVICE_CATALOG = '8286720fbe4941e69fa8241723bb02df'
        self.UUID_TOKEN_UNSCOPED = '731f903721c14827be7b2dc912af7776'
        self.UUID_TOKEN_BIND = '3fc54048ad64405c98225ce0897af7c5'
        self.UUID_TOKEN_UNKNOWN_BIND = '8885fdf4d42e4fb9879e6379fa1eaf48'
        self.VALID_DIABLO_TOKEN = 'b0cf19b55dbb4f20a6ee18e6c6cf1726'
        self.v3_UUID_TOKEN_DEFAULT = '5603457654b346fdbb93437bfe76f2f1'
        self.v3_UUID_TOKEN_UNSCOPED = 'd34835fdaec447e695a0a024d84f8d79'
        self.v3_UUID_TOKEN_DOMAIN_SCOPED = 'e8a7b63aaa4449f38f0c5c05c3581792'
        self.v3_UUID_TOKEN_BIND = '2f61f73e1c854cbb9534c487f9bd63c2'
        self.v3_UUID_TOKEN_UNKNOWN_BIND = '7ed9781b62cd4880b8d8c6788ab1d1e2'

        revoked_token = self.REVOKED_TOKEN
        if isinstance(revoked_token, str):
            revoked_token = revoked_token.encode('utf-8')
        self.REVOKED_TOKEN_HASH = utils.hash_signed_token(revoked_token)
        self.REVOKED_TOKEN_HASH_SHA256 = utils.hash_signed_token(revoked_token,
                                                                 mode='sha256')
        self.REVOKED_TOKEN_LIST = (
            {'revoked': [{'id': self.REVOKED_TOKEN_HASH,
                          'expires': timeutils.utcnow()}]})
        self.REVOKED_TOKEN_LIST_JSON = jsonutils.dumps(self.REVOKED_TOKEN_LIST)

        revoked_v3_token = self.REVOKED_v3_TOKEN
        if isinstance(revoked_v3_token, str):
            revoked_v3_token = revoked_v3_token.encode('utf-8')
        self.REVOKED_v3_TOKEN_HASH = utils.hash_signed_token(revoked_v3_token)
        hash = utils.hash_signed_token(revoked_v3_token, mode='sha256')
        self.REVOKED_v3_TOKEN_HASH_SHA256 = hash
        self.REVOKED_v3_TOKEN_LIST = (
            {'revoked': [{'id': self.REVOKED_v3_TOKEN_HASH,
                          'expires': timeutils.utcnow()}]})
        self.REVOKED_v3_TOKEN_LIST_JSON = jsonutils.dumps(
            self.REVOKED_v3_TOKEN_LIST)

        revoked_token_pkiz = self.REVOKED_TOKEN_PKIZ
        if isinstance(revoked_token_pkiz, str):
            revoked_token_pkiz = revoked_token_pkiz.encode('utf-8')
        self.REVOKED_TOKEN_PKIZ_HASH = utils.hash_signed_token(
            revoked_token_pkiz)
        revoked_v3_token_pkiz = self.REVOKED_v3_TOKEN_PKIZ
        if isinstance(revoked_v3_token_pkiz, str):
            revoked_v3_token_pkiz = revoked_v3_token_pkiz.encode('utf-8')
        self.REVOKED_v3_PKIZ_TOKEN_HASH = utils.hash_signed_token(
            revoked_v3_token_pkiz)

        self.REVOKED_TOKEN_PKIZ_LIST = (
            {'revoked': [{'id': self.REVOKED_TOKEN_PKIZ_HASH,
                          'expires': timeutils.utcnow()},
                         {'id': self.REVOKED_v3_PKIZ_TOKEN_HASH,
                          'expires': timeutils.utcnow()},
                         ]})
        self.REVOKED_TOKEN_PKIZ_LIST_JSON = jsonutils.dumps(
            self.REVOKED_TOKEN_PKIZ_LIST)

        self.SIGNED_TOKEN_SCOPED_KEY = cms.cms_hash_token(
            self.SIGNED_TOKEN_SCOPED)
        self.SIGNED_TOKEN_UNSCOPED_KEY = cms.cms_hash_token(
            self.SIGNED_TOKEN_UNSCOPED)
        self.SIGNED_v3_TOKEN_SCOPED_KEY = cms.cms_hash_token(
            self.SIGNED_v3_TOKEN_SCOPED)

        self.SIGNED_TOKEN_SCOPED_PKIZ_KEY = cms.cms_hash_token(
            self.SIGNED_TOKEN_SCOPED_PKIZ)
        self.SIGNED_TOKEN_UNSCOPED_PKIZ_KEY = cms.cms_hash_token(
            self.SIGNED_TOKEN_UNSCOPED_PKIZ)
        self.SIGNED_v3_TOKEN_SCOPED_PKIZ_KEY = cms.cms_hash_token(
            self.SIGNED_v3_TOKEN_SCOPED_PKIZ)

        self.INVALID_SIGNED_TOKEN = (
            "MIIAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
            "BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB"
            "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC"
            "DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD"
            "EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE"
            "FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
            "0000000000000000000000000000000000000000000000000000000000000000"
            "1111111111111111111111111111111111111111111111111111111111111111"
            "2222222222222222222222222222222222222222222222222222222222222222"
            "3333333333333333333333333333333333333333333333333333333333333333"
            "4444444444444444444444444444444444444444444444444444444444444444"
            "5555555555555555555555555555555555555555555555555555555555555555"
            "6666666666666666666666666666666666666666666666666666666666666666"
            "7777777777777777777777777777777777777777777777777777777777777777"
            "8888888888888888888888888888888888888888888888888888888888888888"
            "9999999999999999999999999999999999999999999999999999999999999999"
            "0000000000000000000000000000000000000000000000000000000000000000")

        self.INVALID_SIGNED_PKIZ_TOKEN = (
            "PKIZ_AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
            "BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB"
            "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC"
            "DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD"
            "EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE"
            "FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
            "0000000000000000000000000000000000000000000000000000000000000000"
            "1111111111111111111111111111111111111111111111111111111111111111"
            "2222222222222222222222222222222222222222222222222222222222222222"
            "3333333333333333333333333333333333333333333333333333333333333333"
            "4444444444444444444444444444444444444444444444444444444444444444"
            "5555555555555555555555555555555555555555555555555555555555555555"
            "6666666666666666666666666666666666666666666666666666666666666666"
            "7777777777777777777777777777777777777777777777777777777777777777"
            "8888888888888888888888888888888888888888888888888888888888888888"
            "9999999999999999999999999999999999999999999999999999999999999999"
            "0000000000000000000000000000000000000000000000000000000000000000")

        # JSON responses keyed by token ID
        self.TOKEN_RESPONSES = {
            self.UUID_TOKEN_DEFAULT: {
                'access': {
                    'token': {
                        'id': self.UUID_TOKEN_DEFAULT,
                        'expires': '2999-01-01T00:00:10.000123Z',
                        'tenant': {
                            'id': 'tenant_id1',
                            'name': 'tenant_name1',
                        },
                    },
                    'user': {
                        'id': 'user_id1',
                        'name': 'user_name1',
                        'roles': [
                            {'name': 'role1'},
                            {'name': 'role2'},
                        ],
                    },
                    'serviceCatalog': {}
                },
            },
            self.VALID_DIABLO_TOKEN: {
                'access': {
                    'token': {
                        'id': self.VALID_DIABLO_TOKEN,
                        'expires': '2999-01-01T00:00:10.000123Z',
                        'tenantId': 'tenant_id1',
                    },
                    'user': {
                        'id': 'user_id1',
                        'name': 'user_name1',
                        'roles': [
                            {'name': 'role1'},
                            {'name': 'role2'},
                        ],
                    },
                },
            },
            self.UUID_TOKEN_UNSCOPED: {
                'access': {
                    'token': {
                        'id': self.UUID_TOKEN_UNSCOPED,
                        'expires': '2999-01-01T00:00:10.000123Z',
                    },
                    'user': {
                        'id': 'user_id1',
                        'name': 'user_name1',
                        'roles': [
                            {'name': 'role1'},
                            {'name': 'role2'},
                        ],
                    },
                },
            },
            self.UUID_TOKEN_NO_SERVICE_CATALOG: {
                'access': {
                    'token': {
                        'id': 'valid-token',
                        'expires': '2999-01-01T00:00:10.000123Z',
                        'tenant': {
                            'id': 'tenant_id1',
                            'name': 'tenant_name1',
                        },
                    },
                    'user': {
                        'id': 'user_id1',
                        'name': 'user_name1',
                        'roles': [
                            {'name': 'role1'},
                            {'name': 'role2'},
                        ],
                    }
                },
            },
            self.UUID_TOKEN_BIND: {
                'access': {
                    'token': {
                        'bind': {'kerberos': self.KERBEROS_BIND},
                        'id': self.UUID_TOKEN_BIND,
                        'expires': '2999-01-01T00:00:10.000123Z',
                        'tenant': {
                            'id': 'tenant_id1',
                            'name': 'tenant_name1',
                        },
                    },
                    'user': {
                        'id': 'user_id1',
                        'name': 'user_name1',
                        'roles': [
                            {'name': 'role1'},
                            {'name': 'role2'},
                        ],
                    },
                    'serviceCatalog': {}
                },
            },
            self.UUID_TOKEN_UNKNOWN_BIND: {
                'access': {
                    'token': {
                        'bind': {'FOO': 'BAR'},
                        'id': self.UUID_TOKEN_UNKNOWN_BIND,
                        'expires': '2999-01-01T00:00:10.000123Z',
                        'tenant': {
                            'id': 'tenant_id1',
                            'name': 'tenant_name1',
                        },
                    },
                    'user': {
                        'id': 'user_id1',
                        'name': 'user_name1',
                        'roles': [
                            {'name': 'role1'},
                            {'name': 'role2'},
                        ],
                    },
                    'serviceCatalog': {}
                },
            },
            self.v3_UUID_TOKEN_DEFAULT: {
                'token': {
                    'expires_at': '2999-01-01T00:00:10.000123Z',
                    'methods': ['password'],
                    'user': {
                        'id': 'user_id1',
                        'name': 'user_name1',
                        'domain': {
                            'id': 'domain_id1',
                            'name': 'domain_name1'
                        }
                    },
                    'project': {
                        'id': 'tenant_id1',
                        'name': 'tenant_name1',
                        'domain': {
                            'id': 'domain_id1',
                            'name': 'domain_name1'
                        }
                    },
                    'roles': [
                        {'name': 'role1', 'id': 'Role1'},
                        {'name': 'role2', 'id': 'Role2'},
                    ],
                    'catalog': {}
                }
            },
            self.v3_UUID_TOKEN_UNSCOPED: {
                'token': {
                    'expires_at': '2999-01-01T00:00:10.000123Z',
                    'methods': ['password'],
                    'user': {
                        'id': 'user_id1',
                        'name': 'user_name1',
                        'domain': {
                            'id': 'domain_id1',
                            'name': 'domain_name1'
                        }
                    }
                }
            },
            self.v3_UUID_TOKEN_DOMAIN_SCOPED: {
                'token': {
                    'expires_at': '2999-01-01T00:00:10.000123Z',
                    'methods': ['password'],
                    'user': {
                        'id': 'user_id1',
                        'name': 'user_name1',
                        'domain': {
                            'id': 'domain_id1',
                            'name': 'domain_name1'
                        }
                    },
                    'domain': {
                        'id': 'domain_id1',
                        'name': 'domain_name1',
                    },
                    'roles': [
                        {'name': 'role1', 'id': 'Role1'},
                        {'name': 'role2', 'id': 'Role2'},
                    ],
                    'catalog': {}
                }
            },
            self.SIGNED_TOKEN_SCOPED_KEY: {
                'access': {
                    'token': {
                        'id': self.SIGNED_TOKEN_SCOPED_KEY,
                        'expires': '2999-01-01T00:00:10.000123Z',
                    },
                    'user': {
                        'id': 'user_id1',
                        'name': 'user_name1',
                        'tenantId': 'tenant_id1',
                        'tenantName': 'tenant_name1',
                        'roles': [
                            {'name': 'role1'},
                            {'name': 'role2'},
                        ],
                    },
                },
            },
            self.SIGNED_TOKEN_UNSCOPED_KEY: {
                'access': {
                    'token': {
                        'id': self.SIGNED_TOKEN_UNSCOPED_KEY,
                        'expires': '2999-01-01T00:00:10.000123Z',
                    },
                    'user': {
                        'id': 'user_id1',
                        'name': 'user_name1',
                        'roles': [
                            {'name': 'role1'},
                            {'name': 'role2'},
                        ],
                    },
                },
            },
            self.SIGNED_v3_TOKEN_SCOPED_KEY: {
                'token': {
                    'expires_at': '2999-01-01T00:00:10.000123Z',
                    'methods': ['password'],
                    'user': {
                        'id': 'user_id1',
                        'name': 'user_name1',
                        'domain': {
                            'id': 'domain_id1',
                            'name': 'domain_name1'
                        }
                    },
                    'project': {
                        'id': 'tenant_id1',
                        'name': 'tenant_name1',
                        'domain': {
                            'id': 'domain_id1',
                            'name': 'domain_name1'
                        }
                    },
                    'roles': [
                        {'name': 'role1'},
                        {'name': 'role2'}
                    ],
                    'catalog': {}
                }
            },
            self.v3_UUID_TOKEN_BIND: {
                'token': {
                    'bind': {'kerberos': self.KERBEROS_BIND},
                    'methods': ['password'],
                    'expires_at': '2999-01-01T00:00:10.000123Z',
                    'user': {
                        'id': 'user_id1',
                        'name': 'user_name1',
                        'domain': {
                            'id': 'domain_id1',
                            'name': 'domain_name1'
                        }
                    },
                    'project': {
                        'id': 'tenant_id1',
                        'name': 'tenant_name1',
                        'domain': {
                            'id': 'domain_id1',
                            'name': 'domain_name1'
                        }
                    },
                    'roles': [
                        {'name': 'role1', 'id': 'Role1'},
                        {'name': 'role2', 'id': 'Role2'},
                    ],
                    'catalog': {}
                }
            },
            self.v3_UUID_TOKEN_UNKNOWN_BIND: {
                'token': {
                    'bind': {'FOO': 'BAR'},
                    'expires_at': '2999-01-01T00:00:10.000123Z',
                    'methods': ['password'],
                    'user': {
                        'id': 'user_id1',
                        'name': 'user_name1',
                        'domain': {
                            'id': 'domain_id1',
                            'name': 'domain_name1'
                        }
                    },
                    'project': {
                        'id': 'tenant_id1',
                        'name': 'tenant_name1',
                        'domain': {
                            'id': 'domain_id1',
                            'name': 'domain_name1'
                        }
                    },
                    'roles': [
                        {'name': 'role1', 'id': 'Role1'},
                        {'name': 'role2', 'id': 'Role2'},
                    ],
                    'catalog': {}
                }
            },
        }
        self.TOKEN_RESPONSES[self.SIGNED_TOKEN_SCOPED_PKIZ_KEY] = (
            self.TOKEN_RESPONSES[self.SIGNED_TOKEN_SCOPED_KEY])
        self.TOKEN_RESPONSES[self.SIGNED_TOKEN_UNSCOPED_PKIZ_KEY] = (
            self.TOKEN_RESPONSES[self.SIGNED_TOKEN_UNSCOPED_KEY])
        self.TOKEN_RESPONSES[self.SIGNED_v3_TOKEN_SCOPED_PKIZ_KEY] = (
            self.TOKEN_RESPONSES[self.SIGNED_v3_TOKEN_SCOPED_KEY])

        self.JSON_TOKEN_RESPONSES = dict([(k, jsonutils.dumps(v)) for k, v in
                                          self.TOKEN_RESPONSES.items()])


EXAMPLES_RESOURCE = testresources.FixtureResource(Examples())


class Deprecations(fixtures.Fixture):
    def setUp(self):
        super(Deprecations, self).setUp()

        # If keystoneclient calls any deprecated function this will raise an
        # exception.
        warnings.filterwarnings('error', category=DeprecationWarning,
                                module='^keystoneclient\\.')
        warnings.filterwarnings('ignore', category=DeprecationWarning,
                                module='^debtcollector\\.')
        self.addCleanup(warnings.resetwarnings)

    def expect_deprecations(self):
        """Call this if the test expects to call deprecated function."""
        warnings.resetwarnings()
        warnings.filterwarnings('ignore', category=DeprecationWarning,
                                module='^keystoneclient\\.')
        warnings.filterwarnings('ignore', category=DeprecationWarning,
                                module='^debtcollector\\.')

    @contextlib.contextmanager
    def expect_deprecations_here(self):
        warnings.resetwarnings()
        warnings.filterwarnings('ignore', category=DeprecationWarning,
                                module='^keystoneclient\\.')
        warnings.filterwarnings('ignore', category=DeprecationWarning,
                                module='^debtcollector\\.')
        yield
        warnings.resetwarnings()
        warnings.filterwarnings('error', category=DeprecationWarning,
                                module='^keystoneclient\\.')
        warnings.filterwarnings('ignore', category=DeprecationWarning,
                                module='^debtcollector\\.')
